<?
/**
 * When including this page, either GET['uid'] or POST['uid'] or $userUID must be set.  This indicates the UID of the
 * patient or employee for whom to show messages.  The $showFirst variable can be set if the user wishes the first
 * secure message in the list to be automatically displayed.  The $messageToShow variable can alternately be used to
 * show a message with a specific ID.
 */
require_once('includes/conf.php');
enforcedSessionStart();

require_once('secure_messages.php');
require_once('includes/portalutils.php');

$userMode = $_SESSION['userMode'];
$loggedIn = false;
$uid = "";
$children = array();
if ($userMode == EMPLOYEE_MODE) {
	$employee = getEmployee(false);
	$uid = $employee->getFieldValue('uid');
	$loggedIn = ($employee != null && isEmployeeLoggedIn($employee));
} else if ($userMode == PATIENT_MODE) {
	$patient = getPatient(false);
	$uid = $patient->getFieldValue('uid');
	$loggedIn = ($patient != null && isLoggedIn($patient));
	$children = getAllChildUIDs();
}

$providedUid = $_GET['uid'];
if (!isset($providedUid)) {
	$providedUid = $_POST['uid'];
}
if (!isset($providedUid)) {
	$providedUid = $userUID;
}
if ($loggedIn) {
	if (!isset($providedUid)) {
		//No UID set - the patient/employee came here via an unexpected process.
		$messagesLink = ($userMode == EMPLOYEE_MODE ? "employee_messages.php" : "patient_messages.php");
		echo "User ID not set.  You must come to this page from the <a href=\"$messagesLink\" class=\"basicLink\">secure messages page</a>.";
	} else {
		$canView = ($providedUid == $uid);
		if (!$canView) {
			//Check to see if the message is for one of the user's children.
			if (isset($children)) {
				foreach ($children as $child) {
					$canView |= ($providedUid == $child);
				}
			}
		}
		if ($canView) {
			//User UID set - load the message.
			try {
				$messages = getSecureMessages($providedUid);
				//echo dump($messages);
				//TODO - remove these debug lines
				/* $soapClient = Backend::getConnection('cec_messaging');
				echo "\n\nRequest:\n" . $soapClient->__getLastRequestHeaders() . "\n\n" . $soapClient->__getLastRequest() . "\n\n" . $soapClient->__getLastResponse(); */
				echo getSecureMessagesHTML($providedUid, $messages, $messageToShow, $showFirst);
			} catch (Exception $e) {
				echo "Error displaying the message.  Please try again.  The IT department has been informed of the issue and will work to resolve it as quickly as possible.";
				//TODO - remove these debug lines
				/* $soapClient = Backend::getConnection('cec_messaging');
				echo "<pre>\nRequest:\n" . $soapClient->__getLastRequestHeaders() . "\n\n" . $soapClient->__getLastRequest() . "</pre>"; */
			}
		} else {
			$messagesLink = ($userMode == EMPLOYEE_MODE ? "employee_messages.php" : "patient_messages.php");
			echo "You cannot view messages for the selected patient.  Please go to the <a href=\"$messagesLink\" class=\"basicLink\">secure messages page</a>.";
		}
	}
} else {
	//Patient/employee not logged in.
	$loginLink = ($userMode == EMPLOYEE_MODE ? "employee_login.php" : "login.php");
	echo "You are not logged in.  Please <a href=\"$loginLink\" class=\"basicLink\">log into</a> the portal before coming to this page.";
}
?>