<?php

# Parameterised query implementation for MySQL (similar PostgreSQL's PHP function pg_query_params)
# Example: mysql_query_params( "SELECT * FROM my_table WHERE col1=$1 AND col2=$2", array( 42, "It's ok" ) );

if( !function_exists( 'mysql_query_params' ) ) {

		function mysql_query_params__callback( $at ) {
				global $mysql_query_params__parameters;
				return $mysql_query_params__parameters[ $at[1]-1 ];
		}

		function mysql_query_params( $query, $parameters=array(), $database=false ) {

				// Escape parameters as required & build parameters for callback function
				global $mysql_query_params__parameters;
				foreach( $parameters as $k=>$v )
						$parameters[$k] = ( is_int( $v ) ? $v : ( NULL===$v ? 'NULL' : "'".mysql_real_escape_string( $v )."'" ) );
				$mysql_query_params__parameters = $parameters;

				// Call using mysql_query
				if( false===$database )
						return mysql_query( preg_replace_callback( '/\$([0-9]+)/', 'mysql_query_params__callback', $query ) );
				else    return mysql_query( preg_replace_callback( '/\$([0-9]+)/', 'mysql_query_params__callback', $query ), $database );

		}
}

if( !function_exists( 'pg_query_parameters' ) ) {

		function pg_query_parameters__callback( $at ) {
				global $pg_query_parameters__parameters;
				return $pg_query_parameters__parameters[ $at[1]-1 ];
		}

		function pg_query_parameters( $query, $parameters=array(), $database=false ) {

				// Escape parameters as required & build parameters for callback function
				global $pg_query_parameters__parameters;
				foreach( $parameters as $k=>$v )
						$parameters[$k] = ( is_int( $v ) ? $v : ( NULL===$v ? 'NULL' : "'".pg_escape_string( $v )."'" ) );
				$pg_query_parameters__parameters = $parameters;

				// Call using pg_query
				if( false===$database )
						return pg_query( preg_replace_callback( '/\$([0-9]+)/', 'pg_query_parameters__callback', $query ) );
				else    return pg_query( preg_replace_callback( '/\$([0-9]+)/', 'pg_query_parameters__callback', $query ), $database );

		}
}
		
class DatabaseObject
{
	private $_verbose = 0;
	private $_supported_types = array('postgresql', 'mysql');
	//set up references
	private $_object_references;
	private $_type;
	
	public function __construct($type, $verbose=0)
	{
		if ($verbose == 1)
		{
			$this->_verbose = 1;
			echo 'VERBOSE MODE ON : DatabaseObject<br />';
		}
		
		if (in_array(strtolower($type), $this->_supported_types))
		{
			$this->set_up_references();
			$this->_type = $type;
		}
		else 
		{
			echo 'Invalid Database Type - Please refer to documentation<br />';
			exit;
		}
	}
	
	private function set_up_references()
	{
		$this->_object_references['postgresql'] = 'PostgreSQL_connection';
		$this->_object_references['mysql'] = 'MYSQL_connection';
	}
	
	public function get_object()
	{
		$new_db_object = new $this->_object_references[$this->_type];
		if ($this->_verbose)
		{
			$new_db_object->verbose();
			echo 'Successful Creation of '. $this->_object_references[$this->_type]. '<br />';
		}
			
		return $new_db_object;
	}
	
	public function __destruct()
	{
		if ($this->_verbose)
			echo 'SUCCESSFULLY destructed DatabaseObject<br />';
	}
}

abstract class DBQuery
{
 	protected $_verbose;
		
	abstract public function __construct($verbose=0);
	
	abstract public function connect($username, $password, $db, $server='localhost', $port=5432);
	
	abstract public function query($sql);
	
	abstract public function query_params($sql, $params);
	
	abstract public function numrows($result);
	
	abstract public function fetch_assoc($result);
	
	abstract public function fetch_array($result);
	
	
	public function verbose()
	{
		$this->_verbose = 1;
	}
	
	abstract function __destruct();
	
}

//***************************************************************************************
//******************* P o s t g r e s S Q L _ C o n n e c t i o n ***********************
//***************************************************************************************

class PostgreSQL_Connection extends DBQuery
{
	private $_connection;
	
	public function __construct($verbose=0)
	{
		$this->_verbose = $verbose;
		if ($this->_verbose == 1)
			echo 'VERBOSE MODE ON<br />';
	}
	
	public function connect($username, $password, $db, $server='localhost', $port=5432)
	{
		$connection = pg_connect("host=$server port=$port dbname=$db user=$username password=$password");
		if (!$connection) {
	  		echo "An error occured with the connection.\n";
			exit;
		}
		else 
		{
			$this->_connection = $connection;
			if ($this->_verbose)
				echo 'Successfully connected as '. $username .' to '. $db .'<br />';
		}
	}
	
	public function query($sql)
	{
		if ($this->_verbose)
		{
			echo 'Executing Query: '. $sql .'<br />';
			$result = pg_query($sql) or die (pg_last_error());
		}
		else
			$result = pg_query($sql);
		
		return $result;	
	}
	
	public function query_params($sql, $params) {
		//Parameterized query for safety
		if ($this->_verbose)
		{
			echo 'Executing Query: ' . $sql . ' (' . $params . ')<br />';
			$result = pg_query_parameters($sql, $params) or die (pg_last_error());
		}
		else
			$result = pg_query_parameters($sql, $params);
		
		return $result;	
	}
	
	public function numrows($result)
	{
		if ($this->_verbose)
			echo "Performing pg_num_rows<br />";
		return pg_num_rows($result);
	}
	
	public function fetch_array($result)
	{
		if ($this->_verbose)
			echo "Performing pg_fetch_array<br />";
		return pg_fetch_array($result);
	}
	
	public function fetch_assoc($result)
	{
		if ($this->_verbose)
			echo "Performing pg_fetch_assoc<br />";
		return pg_fetch_assoc($result);
	}
	
	public function __destruct()
	{
		pg_close($this->_connection);
	}
}

//***************************************************************************************
//******************* M Y S Q L _ C o n n e c t i o n ***********************************
//***************************************************************************************

class MYSQL_Connection extends DBQuery
{
	private $_connection;
	
	public function __construct($verbose=0)
	{
		$this->_verbose = $verbose;
		if ($this->_verbose == 1)
			echo 'VERBOSE MODE ON<br />';
	}
	
	public function connect($username, $password, $db, $server='localhost', $port=5432)
	{
		$connection = mysql_connect($host, $username, $password);
		mysql_select_db($db);
		if (!$connection) {
	  		echo "An error occured with the connection.\n";
			exit;
		}
		else 
		{
			$this->_connection = $connection;
			if ($this->_verbose)
				echo 'Successfully connected as '. $username .' to '. $db .'<br />';
		}
	}
	
	public function query($sql)
	{
		if ($this->_verbose)
		{
			echo 'Executing Query: '. $sql .'<br />';
			$result = mysql_query($sql) or die (mysql_error());
		}
		else
			$result = mysql_query($sql);
		
		return $result;	
	}
	
	public function query_params($sql, $params)
	{
		if ($this->_verbose)
		{
			echo 'Executing Query: ' . $sql . '<br />';
			$result = mysql_query_params($sql, $params) or die (mysql_error());
		}
		else
			$result = mysql_query_params($sql, $params);
		
		return $result;	
	}
	
	public function numrows($result)
	{
		if ($this->_verbose)
			echo "Performing mysql_num_rows<br />";
		return mysql_num_rows($result);
	}
	
	public function fetch_array($result)
	{
		if ($this->_verbose)
			echo "Performing mysql_fetch_array<br />";
		return mysql_fetch_array($result);
	}
	
	public function fetch_assoc($result)
	{
		if ($this->_verbose)
			echo "Performing mysql_fetch_assoc<br />";
		return mysql_fetch_assoc($result);
	}
	
	public function __destruct()
	{
		mysql_close($this->_connection);
	}
}


?>
