<?

require_once('includes.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/preload.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/patient.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/dailypatientlog.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/superbill.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/appointment.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/insurance.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/invalidelement.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/query.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/request.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/emrdatacenter/healthquery.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/emrdatacenter/emrresultfactory.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/dateformatter.php');

/**
 * Attempts to log the message, and returns whether or not the log was actually written
 */
function klog($message, $logLevel) {
	$maxAttempts = 10;
	$logWritten = false;
	$currentLog = 0;
	while ((!$logWritten) && $currentLog < $maxAttempts) {
		$log = new KLogger(dirname(__FILE__) . "/../../log/patientportal" . ($currentLog == 0 ? "" : $currentLog) . ".log" , KLogger::INFO);
		if ($log->Log_Status != KLogger::OPEN_FAILED) {
			echo "Logging<br>\n";
			$log->Log($message, $logLevel);
			$logWritten = true;
		}
		$currentLog++;
	}
	return $logWritten;
}

function generateUID() {
	$request = new Request('UID');
	return $request->run();
}

/**
 * Get insurances arranged for patients to choose.  The first is actually private pay,
 * since this should be presented to users first (so they don't have to hunt for it).
 */
function getInsuranceChoices($dbs=null) {
	$insurances = getAllInsurances($dbs);
	$indexToRemove = -1;
	$privatePayIns = null;
	foreach ($insurances as $index=>$insurance) {
		$uid = $insurance->getFieldValue('uid');
		if ($uid == '0') {
			$indexToRemove = $index;
			$privatePayIns = $insurance;
			break;
		}
	}
	//Add private pay to the beginning if possible
	if ($indexToRemove >= 0 && isset($privatePayIns)) {
		array_splice($insurances, $indexToRemove, 1);
		array_unshift($insurances, $privatePayIns);
	}
	$pleaseSelect = new Insurance(null);
	array_unshift($insurances, $pleaseSelect);
	return $insurances;
}

function getAllInsurances($dbs=null) {
	if ($dbs == null) {
		$dbs = array(DEFAULT_SERVER);
	}
	$query = new Query('INSURANCE_QUERY', $dbs);
	$query->addDataField('*');
	$query->addQualifier('available', '=', 'TRUE');
	$query->orderBy('name', 'asc');
	return $query->run();
	//Just call the preload function
	//(This uses both servers, and only fills some fields, and is thus not really desired)
	//return getAllInsurance();
}

/**
 * Tries to load the patient from the database and, if it fails, optionally sends the user back to a given location
 */
function getPatientFromDB($db, $uid, $backOnFailure, $backLocation) {
	if (!isset($uid) || !isset($db)) {
		if ($backOnFailure) {
			$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
			header("location:$backLocation");
		}
	} else {
		//Construct the Patient object from the UID
		return new Patient($db, $uid);
	}
	return null;
}

function getPatient($backOnFailure=true) {
	return getPatientFromDB($_SESSION['patient_master_db'], $_SESSION['patient'], $backOnFailure, "login.php");
}

/**
 * $backOnFailure determines whether or not to go back to the patient details page if the patient was not found
 */
function getSchedulePatient($backOnFailure=true) {
	return getPatientFromDB($_SESSION['schedule_patient_db'], $_SESSION['schedule_patient'], $backOnFailure, "schedule_start.php");
}

/**
 * Returns the patient used to reserve slots in the scheduler.
 */
function getReservedPatient($db, $backOnFailure=false, $backLocation=null) {
	return getPatientFromDB($db, RESERVED_PATIENT, $backOnFailure, $backLocation);
}

function getPatientLink($patient) {
	$query = new HealthQuery('GetToken', $patient->getFieldValue('uid'));
	$result = $query->run();
	return EMRResultFactory::getSinglePayload($result);
}

function getSuperbill() {
	$superbill = null;
	//If post's 'superbill' variable is set, override any previous session superbill
	if (isset($_POST['superbill']) && isset($_POST['superbill_db'])) {
		setSuperbill($_POST['superbill'], $_POST['superbill_db']);
	}
	if (!isset($_SESSION['superbill']) || !isset($_SESSION['superbill_db'])) {
		//Neither session nor post data has the superbill - go back to superbill selection
		if (isLoggedIn(getPatient())) {
			//User is logged in - let them choose an invoice from a list
			header("location:choose_invoice.php");
		} else {
			//User is not logged in - they need to enter superbill manually or log in first
			$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
			header("location:login.php");
		}
	} else {		
		//Load the superbill
		$superbill = new Superbill($_SESSION['superbill_db']);
		$superbill->load($_SESSION['superbill']);
	}
	return $superbill;
}

/*
 * Checks that neither is false and that there is a match (useful for things like strtotime(),
 * which returns false if it failed to parse.
 */
function isNonFalseMatch($a, $b) {
	return ($a && $b && ($a == $b));
}

/*
 * Logs the user in if the password is correct.  Returns whether or not authentication was successful.
 */
function logIn($patient, $password) {
	if (passwordMatch($patient, $password)) {
		setPatient($patient);
		setSessionPassword($password);
		unset($_SESSION['creditcard']);
		unset($_SESSION['superbill']);
		return true;
	}
	return false;
}

function setSessionPassword($password) {
	$_SESSION['password'] = md5($password);
}

function passwordMatch($patient, $password) {
	$pass = md5($password);
	return ($pass == $patient->getFieldValue('password'));
}

function setPatient($patient) {
	$dbMaster = null;
	//Can take either a Patient object or a UID String
	if (is_string($patient)) {
		$_SESSION['patient'] = $patient;
	} else {
		$_SESSION['patient'] = $patient->getFieldValue('uid');
		$dbMaster = $patient->getDatabaseName();
	}
	
	//Clear out and reset the patient database mapping.
	$_SESSION['patient_dbs'] = array();
	
	$subMaster = calculateDbEntries($_SESSION['patient']);
	if (!isset($dbMaster)) {
		//Only set the master as the first database found if we didn't already have one
		$dbMaster = $subMaster;
	}
	
	foreach (searchForChildren($_SESSION['patient']) as $child) {
		calculateDbEntries($child->getFieldValue('uid'));
	}
	
	//Set the database that is the master (controls demographic data, etc.) for the patient
	$_SESSION['patient_master_db'] = $dbMaster;
}

/*
 * Calculates and sets entries in the database map for the patient provided.
 * Returns the master database (the first database that had the patient)
 */
function calculateDbEntries($pat) {
	//See which databases have that patient in them
	$dbMaster = null;
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('uid', '=', $pat);
	$results = $query->getDBSeparatedResults();
	$dbs = array();
	foreach ($results as $db=>$result) {
		if (isset($result) && count($result) > 0) {
			$dbs[] = $db;
			if (!isset($dbMaster)) {
				$dbMaster = $db;
			}
		}
	}
	//Add mapping for ['patient_dbs'][$pat] to the array calculated
	$_SESSION['patient_dbs'][$pat] = $dbs;
}

/*
 * Takes a plain text password and performs the appropriate encryption, then sets the patient's password
 */
function setPatientPassword($patient, $password) {
	$patient->setFieldValue('password', md5($password));
}

/*
 * Takes a plain text password and converts it into an encrypted temporary password
 */
function setPatientTempPassword($patient, $password) {
	$patient->setFieldValue('password', md5(TEMP_PASSWORD_PREFIX . $password));
}

function setSuperbill($superbill, $superbillDatabase) {
	$_SESSION['superbill'] = $superbill;
	$_SESSION['superbill_db'] = $superbillDatabase;
}

function isLoggedIn($patient) {
	return isset($patient) && isset($_SESSION['password']) && ($_SESSION['password'] == $patient->getFieldValue('password'));
}

function userExists($username) {
	return getPatientByUsername($username) != null;
}

/*
 * Returns all patients that are listed under this account (patient themselves and all listed children)
 */
function getAllUnderAccount($patient) {
	$patients = getListedChildren($patient);
	//Make sure array of patients is non-null
	if (!isset($patients)) {
		$patients = array();
	}
	//Add the patient themself to the start of the array
	array_unshift($patients, $patient);
	return $patients;
}

/*
 * Returns all patients whose parent is stored as the given patient
 * For speed, only looks for patients that were cached at login (won't refresh until another login)
 */
function getListedChildren($patient) {
	$children = array();
	foreach ($_SESSION['patient_dbs'] as $pat=>$dbs) {
		if ($pat != $patient->getFieldValue('uid') && isset($dbs) && count($dbs) > 0) {
			//It's a child; pull its data from the first available database
			$query = new Query('PATIENT_QUERY', array($dbs[0]));
			$query->addDataField('*');
			$query->addQualifier('uid', '=', $pat);
			$results = $query->run();
			if (isset($results) && count($results) > 0) {
				$children[] = $results[0];
			}
		}
	}
	return $children;
}

function searchForChildren($patient) {
	$uid = $patient;
	if (!is_string($patient)) {
		$uid = $patient->getFieldValue('uid');
	}
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('parent', '=', $uid);
	return uniquify($query->run(), 'uid');
}

function userHasChild($patient) {
	foreach ($_SESSION['patient_dbs'] as $pat=>$dbs) {
		if ($pat == $patient) {
			return true;
		}
	}
	return false;
}

function userCanSeeSuperbill($patient, $superbill) {
	$patientUid = $patient->getFieldValue('uid');
	
	$superbillPatient = getPatientBySuperbill($superbill->getSuperbillNumber());
	$sbPatientUid = $superbillPatient->getFieldValue('uid');
	return (($sbPatientUid == $patientUid) || (userHasChild($superbillPatient->getFieldValue('uid'))));
}

function getPatientByUsername($username) {
	$patient = new Patient(null);
	if ($patient->setFromFirstWhere('username', $username, 'ilike')) {
		//Patient found
		return $patient;
	} else {
		return null;
	}
}

function getPatientsByEmailInfo($email, $bd=null) {
	$df = new DateFormatter('Y-m-d');
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('email', 'ilike', $email);
	if (isset($bd)) {
		$query->addQualifier('birthday', '=', $df->format($bd));
	}
	
	//We want unique patients by uid (not one entry for CEC and one for VC, for example.)
	$patients = $query->run();
	return uniquify($patients, 'uid');
}

function getSuperbillsForPatient($patient, $start=null, $end=null, $order='desc') {
	$uid = $patient->getFieldValue('uid');
	if (!isset($_SESSION['patient_dbs'][$uid])) {
		calculateDbEntries($uid);
	}
	$dbs = $_SESSION['patient_dbs'][$uid];
	
	$query = new Query('DAILYPATIENTLOG', $dbs);
	$query->addDataField('superbill');
	//$query->addDataField('date');
	$query->addQualifier('patient','=',$uid);
	if (isset($start)) {
		$query->addQualifier('date','>=',$start);
	}
	if (isset($end)) {
		$query->addQualifier('date','<=',$end);
	}
	$query->orderBy('date', $order);

	return uniquify($query->run(), 'superbill');
}

function uniquify($list, $uniqueField) {
	$unique = array();
	$found = array();
	foreach ($list as $item) {
		$fieldVal = $item->getFieldValue($uniqueField);
		if (!in_array($fieldVal, $found)) {
			$found[] = $fieldVal;
			$unique[] = $item;
		}
	}
	return $unique;
}

function filterByBirthday($patients, $birthday) {
	$specificPatients = array();
	if (isset($patients)) {
		$paramBd = strtotime($birthday);
		foreach ($patients as $patient) {
			if (isNonFalseMatch($paramBd, strtotime($patient->getFieldValue('birthday')))) {
				//Patient has correct birthday
				$specificPatients[] = $patient;
			}
		}
	}
	return $specificPatients;
}

function filterByUserType($patients, $isUser) {
	$specificPatients = array();
	foreach ($patients as $patient) {
		if ($hasUser == ($patient->getFieldValue('username') != null)) {
			//Patient has correct existence of username
			$specificPatients[] = $patient;
		}
	}
	return $specificPatients;
}

function getPatientBySuperbill($superbill) {
	$dpl = new DailyPatientLog(null);
	if ($dpl->setFromFirstWhere('superbill', $superbill)) {
		//DPL found - get the DPL's patient
		return new Patient($dpl->getDatabaseName(), $dpl->getFieldValue('patient'));
	} else {
		return null;
	}
}

function getPatientByTempPassword($pass) {
	$patient = new Patient(null);
	if ($patient->setFromFirstWhere('password', md5(TEMP_PASSWORD_PREFIX . $pass), '=')) {
		//Patient found
		return $patient;
	} else {
		return null;
	}
}

function nameMatches($patient, $firstname, $lastname) {
	return (strtoupper($patient->getFieldValue('firstname')) == strtoupper($firstname) &&
	       strtoupper($patient->getFieldValue('lastname')) == strtoupper($lastname));
}

function googleAction($actionName, $flag) {
	$patient = getPatient();
	$logged_in = isLoggedIn($patient);

	if (!$logged_in) {
		$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
		header('location:login.php');
	} else {
		$patientLink = getPatientLink($patient);
		if (!isset($patientLink) || $patientLink instanceof InvalidElement) {
			//There is no link - allow the user to link themselves with google
			header('location:link_with_google.php');
		} else {
			//There is a session token linked for that patient. Perform appropriate action
			$query = new HealthQuery($flag, $patient->getFieldValue('uid'));
			$query->setPayload($patientLink);
			$results = $query->run();
			$singleResult = EMRResultFactory::getSinglePayload($results);
			
			if (isset($singleResult) && $singleResult instanceof InvalidElement) {
				//If results are an InvalidElement, an error occurred
				header('location:google_failure.php');
			} else {
				//If the results aren't an InvalidElement, action was successful
				$_SESSION['google_action'] = $actionName;
				header('location:link_with_google.php');
			}
		}
	}
}

?>