<?

require_once('includes.php');
require_once(GLOBAL_INCLUDES_DIR . 'xml_to_html.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/dflist.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/preload.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/patient.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/dailypatientlog.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/superbill.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/appointment.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/insurance.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/invalidelement.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/auditlogentry.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/query.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/request.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/emrdatacenter/healthquery.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/emrdatacenter/emrresultfactory.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/ChangeUsernameMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/ChangePasswordMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/findscheduledprovidersmessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetPatientByUsernameMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetPatientUsernameMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/SetPatSignonMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/VerifyPatientCredentialsMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/PatientUsernameExistsMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/dateformatter.php');

/**
 * Attempts to log the message, and returns whether or not the log was actually written
 */
function klog($message, $logLevel) {
	$maxAttempts = 10;
	$logWritten = false;
	$currentLog = 0;
	while ((!$logWritten) && $currentLog < $maxAttempts) {
		$log = new KLogger(dirname(__FILE__) . "/../../log/patientportal" . ($currentLog == 0 ? "" : $currentLog) . ".log" , KLogger::INFO);
		if ($log->Log_Status != KLogger::OPEN_FAILED) {
			$log->Log($message, $logLevel);
			$logWritten = true;
		}
		$currentLog++;
	}
	return $logWritten;
}

/**
 * Returns whether or not the user's session has expired.
 */
function sessionExpired() {
	//If there's nothing in the session, or the only thing is the state array, the session has expired
	return (!isset($_SESSION)) || (!is_array($_SESSION)) || (count($_SESSION) == 0) || (count($_SESSION) == 1 && isset($_SESSION['stateArray']));
}

function generateUID() {
	$request = new Request('UID');
	return $request->run();
}

/**
 * The patient opted out of receiving e-mails.  Audit the event for future reference.
 */
function auditOptOut($db, $patientUID, $locationStamp) {
	//$currentDate = date('Y-m-d');
	//$currentTime = date('H:i:s');
	
	$auditEntry = new AuditLogEntry($db);
	$auditEntry->setFieldValue('uid', generateUID());
	//$auditEntry->setFieldValue('newdata', "PHONE");
	$auditEntry->setFieldValue('record', $patientUID);
	$auditEntry->setFieldValue('reason', "Email Unsubscribe");
	$dbs = unserialize(SERVERS);
	$auditEntry->setFieldValue('userid', $dbs[$db]['employee_uid']);
	$auditEntry->setFieldValue('audit_type', "EMAIL_OPT_OUT");
	$auditEntry->setFieldValue('data_table', "patients");
	$auditEntry->setFieldValue('_column', "preferred_contact");
	//$auditEntry->setFieldValue('date', $currentDate);
	//$auditEntry->setFieldValue('time', $currentTime);
	$auditEntry->setFieldValue('location', $locationStamp);
	$auditEntry->setFieldValue('location_stamp', $locationStamp);
	
	$auditEntry->saveData();
}

/**
 * Get insurances arranged for patients to choose.  The first is actually private pay,
 * since this should be presented to users first (so they don't have to hunt for it).
 */
function getInsuranceChoices($dbs=null) {
	$insurances = getAllInsurances($dbs);
	$indexToRemove = -1;
	$privatePayIns = null;
	foreach ($insurances as $index=>$insurance) {
		$uid = $insurance->getFieldValue('uid');
		if ($uid == '0') {
			$indexToRemove = $index;
			$privatePayIns = $insurance;
			break;
		}
	}
	//Add private pay to the beginning if possible
	if ($indexToRemove >= 0 && isset($privatePayIns)) {
		array_splice($insurances, $indexToRemove, 1);
		array_unshift($insurances, $privatePayIns);
	}
	$pleaseSelect = new Insurance(null);
	array_unshift($insurances, $pleaseSelect);
	return $insurances;
}

function filterSchedulerInsurances($insurances) {
	//These insurance companies are not to be included as options online
	$dontInclude = array('473C8A38E0E42CBD7B07534C8A90DCBB', '91FDFF902ACECD33A24F9A0DAA525F3A', '8107EF51AA633C9E790322AA47C6E5D4',
						'9A2220D0FA5EB5CAA15B5AF7788FF115', '32909B802910EF0E97968CE29580E39B', '49FC441BFB99A4A0B27303567F95123E',
						'A6E6CE75FE9FF60DE3196E6D3AA3C218', 'B3B64A3327F85ED5C92F5C5C42948F12', '74871B600FE41588722B8480BFECB875',
						'9BCBC57A0B121456556D85AC7AEB49A7', '33C08FA9B07308C5D817344039C66EF0');
	$newList = array();
	foreach ($insurances as $insurance) {
		if (!in_array($insurance->getFieldValue('uid'), $dontInclude)) {
			$newList[] = $insurance;
		}
	}
	return $newList;
}

function getAllInsurances($dbs=null) {
	if ($dbs == null) {
		$dbs = array(DEFAULT_SERVER);
	}
	$query = new Query('INSURANCE_QUERY', $dbs);
	$query->addDataField('*');
	$query->addQualifier('available', '=', 'TRUE');
	$query->orderBy('name', 'asc');
	return $query->run();
	//Just call the preload function
	//(This uses both servers, and only fills some fields, and is thus not really desired)
	//return getAllInsurance();
}

function patHasUsername($patient) {
	return (getPatUsername($patient) != null);
}

function getPatUsername($patient) {
	//return ($patient->getFieldValue('username'));
	$msg = new GetPatientUsernameMessage('cec_portal');
	$users = $msg->getUsername($patient->getFieldValue('uid'));
	if ($users == null) {
		return null;
	}
	//There may (should) only be one username.  However, if an array, just use the first.
	return (is_array($users) ? $users[0] : $users);
}
/**
 * Tries to load the patient from the database and, if it fails, optionally sends the user back to a given location
 */
function getPatientFromDB($db, $uid, $backOnFailure, $backLocation) {
	if (!isset($uid) || !isset($db)) {
		if ($backOnFailure) {
			$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
			header("location:$backLocation");
		}
	} else {
		//Construct the Patient object from the UID
		return new Patient($db, $uid);
	}
	return null;
}

function getPatient($backOnFailure=true) {
	return getPatientFromDB($_SESSION['patient_master_db'], $_SESSION['patient'], $backOnFailure, "login.php");
}

/**
 * $backOnFailure determines whether or not to go back to the patient details page if the patient was not found
 */
function getSchedulePatient($backOnFailure=true) {
	return getPatientFromDB($_SESSION['schedule_patient_db'], $_SESSION['schedule_patient'], $backOnFailure, "schedule_start.php");
}

/**
 * Returns the patient used to reserve slots in the scheduler.
 */
function getReservedPatient($db, $backOnFailure=false, $backLocation=null) {
	return getPatientFromDB($db, RESERVED_PATIENT, $backOnFailure, $backLocation);
}

function getPatientLink($patient) {
	$query = new HealthQuery('GetToken', $patient->getFieldValue('uid'));
	$result = $query->run();
	return EMRResultFactory::getSinglePayload($result);
}

function getSuperbill() {
	$superbill = null;
	//If post's 'superbill' variable is set, override any previous session superbill
	if (isset($_POST['superbill']) && isset($_POST['superbill_db'])) {
		setSuperbill($_POST['superbill'], $_POST['superbill_db']);
	}
	if (!isset($_SESSION['superbill']) || !isset($_SESSION['superbill_db'])) {
		//Neither session nor post data has the superbill - go back to superbill selection
		if (isLoggedIn(getPatient())) {
			//User is logged in - let them choose an invoice from a list
			header("location:choose_invoice.php");
		} else {
			//User is not logged in - they need to enter superbill manually or log in first
			$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
			header("location:login.php");
		}
	} else {		
		//Load the superbill
		$superbill = new Superbill($_SESSION['superbill_db']);
		$superbill->load($_SESSION['superbill']);
	}
	return $superbill;
}

/**
 * Gets the first available location from any database, where the uid is the one given
 */
function getLocationByUID($uid) {
	$location = new Location(null);
	if ($uid != null && $location->setFromFirstWhere('uid', $uid, '=', unserialize(SERVER_NAMES), true)) {
		//Location found - return it
		return $location;
	} else {
		return null;
	}
}

/**
 * Gets the first available location from any database, where the location number is the one given
 */
function getLocationByNumber($locationNumber) {
	if ($locationNumber == null) {
		return null;
	}
	$location = new Location(null);
	$query = new Query('LOCATION_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('available', '=', 'TRUE');
	if ($locationNumber == 87) {
		//Dirty hack - we don't want location 87 to trigger "Ballas Outpatient" or one of those others... we want Columbia.
		$query->addQualifier('uid', '=', 'EDFF5152585264BC3D8D451AC0A1AE0A');
	} else if ($locationNumber == 91) {
		//Another dirty hack - like above, but "HHBGC Grand" should be Collinsville
		$query->addQualifier('uid', '=', '3E862FBA0D131BF45E8C0343C96C5413');
	} else if ($locationNumber == 92) {
		//Another dirty hack - like above, but "HHBGC Adams Park" should be Columbia - Rockbridge
		$query->addQualifier('uid', '=', '8E4387C96A950DE8229906B287DC4E01');
	} else if ($locationNumber == 80) {
		//Another dirty hack - like above, but "3F VISION" should be Granite City
		$query->addQualifier('uid', '=', '924C7A82D195D5FDD793286D513E68E0');
	} else if ($locationNumber == 90) {
		$query->addQualifier('uid', '=', '4897E9D4A519D67561D5C960E94C7D5A');
	} else {
		//For all but those specified above, just use the location number and available flag.
		$query->addQualifier('location_number', '=', $locationNumber);
	}
	$results = $query->run();
	if ($results != null && count($results) > 0) {
		//Location found - return it
		return $results[0];
	} else {
		return null;
	}
}

/*
 * Checks that neither is false and that there is a match (useful for things like strtotime(),
 * which returns false if it failed to parse.
 */
function isNonFalseMatch($a, $b) {
	return ($a && $b && ($a == $b));
}

function clearVarsForLogin() {
	//Clear any invoice- or schedule-related fields
	$vars = array('creditcard', 'superbill', 'secure_cc_display');
	foreach ($vars as $var) {
		unset($_SESSION[$var]);
	}
	foreach ($_SESSION as $field=>$value) {
		if (strpos($field, 'schedule_') === 0 && $field != 'location' && $field != 'location_db') {
			unset($_SESSION[$field]);
		}
	}
}

function logIn($username, $password) {
	$patient = getPatientByCredentials($username, $password);
	if ($patient != null) {
		setPatient($patient);
		setSessionPassword($password);
		clearVarsForLogin();
		$_SESSION['userMode'] = PATIENT_MODE;
		return true;
	}
	return false;
}

//Don't do this.  Log in with username and password, not Patient object and password.  See above.
/*
 * Logs the user in if the password is correct.  Returns whether or not authentication was successful.
 */
/*function logIn($patient, $password) {
	if (passwordMatch($patient, $password)) {
		setPatient($patient);
		setSessionPassword($password);
		clearVarsForLogin();
		return true;
	}
	return false;
}*/

function setSessionPassword($password) {
	//Now SESSION['password'] is just a flag.
	$_SESSION['password'] = true;
}

function verifyCredentials($username, $password) {
	$patient = getPatientByCredentials($username, $password);
	return ($patient != null);
}

function getPatientByCredentials($username, $password) {
	$msg = new VerifyPatientCredentialsMessage('cec_portal');
	return $msg->verifyCredentials($username, $password);
}

function setPatient($patient) {
	$dbMaster = null;
	//Can take either a Patient object or a UID String
	if (is_string($patient)) {
		$_SESSION['patient'] = $patient;
	} else {
		$_SESSION['patient'] = $patient->getFieldValue('uid');
		$dbMaster = $patient->getDatabaseName();
	}
	
	//Clear out and reset the patient database mapping.
	$_SESSION['patient_dbs'] = array();
	
	$subMaster = calculateDbEntries($_SESSION['patient']);
	if (!isset($dbMaster)) {
		//Only set the master as the first database found if we didn't already have one
		$dbMaster = $subMaster;
	}
	
	foreach (searchForChildren($_SESSION['patient']) as $child) {
		calculateDbEntries($child->getFieldValue('uid'));
	}
	
	//Set the database that is the master (controls demographic data, etc.) for the patient
	$_SESSION['patient_master_db'] = $dbMaster;
}

/*
 * Calculates and sets entries in the database map for the patient provided.
 * Returns the master database (the first database that had the patient)
 */
function calculateDbEntries($pat) {
	//See which databases have that patient in them
	$dbMaster = null;
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('uid', '=', $pat);
	$results = $query->getDBSeparatedResults();
	$dbs = array();
	foreach ($results as $db=>$result) {
		if (isset($result) && count($result) > 0) {
			$dbs[] = $db;
			if (!isset($dbMaster)) {
				$dbMaster = $db;
			}
		}
	}
	//Add mapping for ['patient_dbs'][$pat] to the array calculated
	$_SESSION['patient_dbs'][$pat] = $dbs;
}

/*
 * Takes a plain text password and performs the appropriate encryption, then sets the patient's password and saves them.
 */
function setPatientPassword($patient, $oldPassword, $password) {
	//Old way - "username" and "password" fields set via Victor
	/* $patient->setFieldValue('password', md5($password));
	$invalid = $patient->saveData();
	return ((!isset($invalid)) || count($invalid) == 0); */
	//Now use Axis2 service instead, per issue #3646
	$msg = new ChangePasswordMessage('cec_portal');
	return $msg->changePassword($patient->getFieldValue('uid'), $oldPassword, $password);
}

/*
 * Change the patient's username to the value provided, then save the patient.
 */
function setPatientUsername($patient, $username) {
	//Old way - "username" and "password" fields set via Victor
	/* $patient->setFieldValue('username', $username);
	$invalid = $patient->saveData();
	return ((!isset($invalid)) || count($invalid) == 0); */
	//Now use Axis2 service instead, per issue #3646
	$msg = new ChangeUsernameMessage('cec_portal');
	return $msg->changeUsername($patient->getFieldValue('uid'), $username);
}

/*
 * Takes a plain text password and converts it into an encrypted temporary password
 */
function setPatientTempLogin($patient, $username, $password) {

	$msg = new SetPatSignonMessage('cec_portal');
	return $msg->setSignon($patient->getFieldValue('uid'), $username, $password);
}

function setSuperbill($superbill, $superbillDatabase) {
	$_SESSION['superbill'] = $superbill;
	$_SESSION['superbill_db'] = $superbillDatabase;
}

function isLoggedIn($patient) {
	return isset($patient) && isset($_SESSION['password']) && $_SESSION['patient'] == $patient->getFieldValue('uid');
}

function userExists($username) {
	$msg = new PatientUsernameExistsMessage('cec_portal');
	return ($msg->usernameExists($username));
}

/*
 * Returns all patients that are listed under this account (patient themselves and all listed children)
 */
function getAllUnderAccount($patient) {
	$patients = getListedChildren($patient);
	//Make sure array of patients is non-null
	if (!isset($patients)) {
		$patients = array();
	}
	//Add the patient themself to the start of the array
	array_unshift($patients, $patient);
	return $patients;
}

/*
 * Returns all patients whose parent is stored as the given patient
 * For speed, only looks for patients that were cached at login (won't refresh until another login)
 */
function getListedChildren($patient) {
	$children = array();
	foreach ($_SESSION['patient_dbs'] as $pat=>$dbs) {
		if ($pat != $patient->getFieldValue('uid') && isset($dbs) && count($dbs) > 0) {
			//It's a child; pull its data from the first available database
			$query = new Query('PATIENT_QUERY', array($dbs[0]));
			$query->addDataField('*');
			$query->addQualifier('uid', '=', $pat);
			$results = $query->run();
			if (isset($results) && count($results) > 0) {
				$children[] = $results[0];
			}
		}
	}
	return $children;
}

function searchForChildren($patient) {
	$uid = $patient;
	if (!is_string($patient)) {
		$uid = $patient->getFieldValue('uid');
	}
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('parent', '=', $uid);
	return uniquify($query->run(), 'uid');
}

function userHasChild($patient) {
	foreach ($_SESSION['patient_dbs'] as $pat=>$dbs) {
		if ($pat == $patient) {
			return true;
		}
	}
	return false;
}

function userCanSeeSuperbill($patient, $superbill) {
	$patientUid = $patient->getFieldValue('uid');
	
	$superbillPatient = getPatientBySuperbill($superbill->getSuperbillNumber());
	$sbPatientUid = $superbillPatient->getFieldValue('uid');
	return (($sbPatientUid == $patientUid) || (userHasChild($superbillPatient->getFieldValue('uid'))));
}

function getPatientByUsername($username) {
	/* $patient = new Patient(null);
	if ($patient->setFromFirstWhere('username', strtolower($username), 'llike')) {
		//Patient found
		return $patient;
	} else {
		return null;
	} */
	$msg = new GetPatientByUsernameMessage('cec_portal');
	return $msg->getPatientByUsername($username);
}

function getPatientsByEmailInfo($email, $bd=null) {
	$df = new DateFormatter('Y-m-d');
	$query = new Query('PATIENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('email', 'llike', strtolower($email));
	if (isset($bd)) {
		$query->addQualifier('birthday', '=', $df->format($bd));
	}
	
	//We want unique patients by uid (not one entry for CEC and one for VC, for example.)
	$patients = $query->run();
	return uniquify($patients, 'uid');
}

function getLastAppointmentForPatient($patient) {
	$uid = $patient->getFieldValue('uid');
	$query = new Query('APPOINTMENT_QUERY', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('patient', '=', $uid);
	$query->orderBy('date', 'desc');
	$query->addLimit(1);
	
	$appointments = $query->run();
	return (($appointments != null && count($appointments) >= 1) ? $appointments[0] : null);
}

function getLastFullExamDPLForPatient($patient) {
	$uid = $patient->getFieldValue('uid');
	$query = new Query('DAILYPATIENTLOG', unserialize(SERVER_NAMES));
	$query->addDataField('*');
	$query->addQualifier('patient', '=', $uid);
	$query->orderBy('date', 'desc');
	
	//Find last exam that was for one of the 4 full exam CPT's
	$dpls = $query->run();
	//Can't do "or" criteria in these queries - the filtering must be done myself.
	$fullExams = array('92004', '92014', 'S0620', 'S0621');
	if ($dpls != null) {
		foreach ($dpls as $dpl) {
			if (in_array($dpl->getFieldValue('exam_type'), $fullExams)) {
				return $dpl;
			}
		}
	}
	return null;
}

function getSuperbillsForPatient($patient, $start=null, $end=null, $order='desc') {
	$uid = $patient->getFieldValue('uid');
	if (!isset($_SESSION['patient_dbs'][$uid])) {
		calculateDbEntries($uid);
	}
	$dbs = $_SESSION['patient_dbs'][$uid];
	
	$query = new Query('DAILYPATIENTLOG', $dbs);
	$query->addDataField('superbill');
	//$query->addDataField('date');
	$query->addQualifier('patient','=',$uid);
	if (isset($start)) {
		$query->addQualifier('date','>=',$start);
	}
	if (isset($end)) {
		$query->addQualifier('date','<=',$end);
	}
	$query->orderBy('date', $order);

	return uniquify($query->run(), 'superbill');
}

/**
 * Returns true if the location has been moved to a new location
 */
function hasNewLocation($locationUID) {
	$movedLocations = unserialize(MOVED_LOCATIONS);
	return array_key_exists($locationUID, $movedLocations);
}

/**
 * Returns the location to which the given location has been moved
 */
function getNewLocation($locationUID) {
	$movedLocations = unserialize(MOVED_LOCATIONS);
	$newLocUID = $movedLocations[$locationUID];
	return getLocationByUID($newLocUID);
}

function getAllProvidersFor($locationUID, $dbs) {
	$query = new Query('PROVIDER_QUERY', $dbs);
	$query->addDataField('uid');
	$query->addDataField('last_name');
	$query->addDataField('first_name');
	$query->addQualifier('location_list', 'like', '%' . $locationUID . '%');
	$query->addQualifier('available', '=', 'TRUE');
	$query->orderBy('last_name', 'asc');
	$query->orderBy('first_name', 'asc');
	
	$results = $query->run();
	if (isset($results) && count($results) > 0 && ($results[0] instanceof Provider)) {
		return $results;
	} else {
		return null;
	}
}

function getScheduledProvidersFor($locationUID, $axisServer) {
	$message = new FindScheduledProvidersMessage($axisServer);
	$startDate = date('Y-m-dP');
	$endMillis = add_date(strtotime($startDate), 0, 2, 0);
	return $message->findScheduledProviders($startDate, date('Y-m-dP', $endMillis), $locationUID);
}

function getLastVisitFor($patient) {
	$query = new Query('APPOINTMENT_QUERY', array($patient->getDatabaseName()));
	$query->addDataField('*');
	$query->addQualifier('patient', '=', $patient->getFieldValue('uid'));
	$query->addQualifier('date', '<=', date('Y-m-d'));
	$query->orderBy('date', 'desc');
	$query->addLimit(1);
	$results = $query->run();
	if (isset($results) && count($results) > 0) {
		return $results[0];
	} else {
		return null;
	}
}

function uniquify($list, $uniqueField) {
	$unique = array();
	$found = array();
	foreach ($list as $item) {
		$fieldVal = $item->getFieldValue($uniqueField);
		if (!in_array($fieldVal, $found)) {
			$found[] = $fieldVal;
			$unique[] = $item;
		}
	}
	return $unique;
}

function filterByBirthday($patients, $birthday) {
	$specificPatients = array();
	if (isset($patients)) {
		$paramBd = strtotime($birthday);
		foreach ($patients as $patient) {
			if (isNonFalseMatch($paramBd, strtotime($patient->getFieldValue('birthday')))) {
				//Patient has correct birthday
				$specificPatients[] = $patient;
			}
		}
	}
	return $specificPatients;
}

function filterByUserType($patients, $isUser) {
	$specificPatients = array();
	foreach ($patients as $patient) {
		if ($hasUser == ($patient->getFieldValue('username') != null)) {
			//Patient has correct existence of username
			$specificPatients[] = $patient;
		}
	}
	return $specificPatients;
}

function getPatientBySuperbill($superbill) {
	$dpl = new DailyPatientLog(null);
	if ($dpl->setFromFirstWhere('superbill', $superbill)) {
		//DPL found - get the DPL's patient
		return new Patient($dpl->getDatabaseName(), $dpl->getFieldValue('patient'));
	} else {
		return null;
	}
}

//This function is no longer necessary.
/* function getPatientByTempPassword($user, $pass) {
	$patient = new Patient(null);
	if ($patient->setFromFirstWhere('password', md5(TEMP_PASSWORD_PREFIX . $pass), '=')) {
		//Patient found
		return $patient;
	} else {
		return null;
	}
} */

function nameMatches($patient, $firstname, $lastname) {
	return (strtoupper($patient->getFieldValue('firstname')) == strtoupper($firstname) &&
	       strtoupper($patient->getFieldValue('lastname')) == strtoupper($lastname));
}

/**
 * Builds an array of enumeration options for welcome sheets suitable for use in xml_to_html and xml_utils.
 */
function buildWelcomeSheetEnumerations() {
	$enumChoices = getEnumChoicesFromFile(dirname(__FILE__) . '/../welcome_sheet_enumeration_fields.xml');
	//Add designations from data fields
	$designationList = DFList::getDFList('PersonTitle');
	$designations = array();
	foreach ($designationList->getDataFields() as $key=>$designation) {
		$designations[] = array('key'=>$key, 'value'=>$designation->getFieldValue('name'));
	}
	$enumChoices['patient__designation'] = $designations;
	//Add preferred languages from data fields
	$languageList = DFList::getDFList('Languages');
	$languages = array();
	foreach ($languageList->getDataFields() as $key=>$language) {
		$languages[] = array('key'=>$language->getFieldValue('name'), 'value'=>$language->getFieldValue('name'));
	}
	$enumChoices['patient__preferred_language'] = $languages;
	return $enumChoices;
}

function googleAction($actionName, $flag) {
	$patient = getPatient();
	$logged_in = isLoggedIn($patient);

	if (!$logged_in) {
		$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
		header('location:login.php');
	} else {
		$patientLink = getPatientLink($patient);
		if (!isset($patientLink) || $patientLink instanceof InvalidElement) {
			//There is no link - allow the user to link themselves with google
			header('location:link_with_google.php');
		} else {
			//There is a session token linked for that patient. Perform appropriate action
			$query = new HealthQuery($flag, $patient->getFieldValue('uid'));
			$query->setPayload($patientLink);
			$results = $query->run();
			$singleResult = EMRResultFactory::getSinglePayload($results);
			
			if (isset($singleResult) && $singleResult instanceof InvalidElement) {
				//If results are an InvalidElement, an error occurred
				header('location:google_failure.php');
			} else {
				//If the results aren't an InvalidElement, action was successful
				$_SESSION['google_action'] = $actionName;
				header('location:link_with_google.php');
			}
		}
	}
}

?>
