<?
	define("MIN_LENGTH", 9);
	define("SPECIAL", "~!@#\\$%\\^&\\*\\(\\)_\\+\\[\\]\\{\\}\\|;,\\.<>\\?");
	/* Password Strength Requirements:
         * [0-9]             # at least one digit
         * [a-z]             # at least one lower case letter
         * [A-Z]             # at least one upper case character
         * [SPECIAL]         # at least one special character
         * .{MIN_LENGTH,}    # a minimum length 
	*/
	
	/**
	 * Output HTML that shows what is required in a password.  Each item is identifiable so it may be modified as requirements are met.
	 * $inputID - The ID of the input element that contains the value to check
	 */
	function showPasswordRequirements($inputID) {
		if ($inputID != null) {
			echo "
				<script type=\"text/javascript\">
					//<![CDATA[
					function updateRequirements() {
						var input = $('#$inputID').val();
						$('.passwordReq').removeClass('boldText');
						$('.passwordReq:not(id=\"passwordSpecialRequired\")').each(function() {
							var str = $(this).html();
							//Strip the ' (not yet met)' - it will be added later if necessary.
							if (str.substr(str.length - 14, 14) == ' (not yet met)') {
								$(this).html(str.substr(0, str.length - 14));
							}
						});
						var element = $('#passwordSpecialRequired');
						var elementHTML = element.html();
						if (elementHTML.substr(46, 14) == ' (not yet met)') {
							element.html(elementHTML.substr(0, 46) + elementHTML.substr(60, elementHTML.length - 60));
						}
						if (input == null || input == '') {
							$('.passwordReq').addClass('boldText');
							addNotMet($('.passwordReq:not(id=\"passwordSpecialRequired\")'), null);
							addNotMet($('#passwordSpecialRequired'), 46);
						} else {
							if (input.length < " . MIN_LENGTH . ") {
								$('#minPasswordLength').addClass('boldText');
								$('#minPasswordLength').html($('#minPasswordLength').html() + ' (not yet met)');
							}
							matchReq(input, /[0-9]/, '#passwordDigitsRequired', null);
							matchReq(input, /[a-z]/, '#passwordLowerCaseRequired', null);
							matchReq(input, /[A-Z]/, '#passwordUpperCaseRequired', null);
							matchReq(input, /[" . SPECIAL . "]/, '#passwordSpecialRequired', 46);
						}
					}
					
					function matchReq(input, expr, warnID, start) {
						var match = input.match(expr);
						if (match == null || match == '') {
							$(warnID).addClass('boldText');
							addNotMet($(warnID), start);
						}
					}
					
					function addNotMet(element, start) {
						if (start == null) {
							element.each(function() {
								$(this).html($(this).html() + ' (not yet met)');
							});
						} else {
							element.html(element.html().substr(0, start) + ' (not yet met)' + element.html().substr(start, element.html().length - start));
						}
					}
					
					$(document).ready(function() {
						$('#$inputID').keyup(updateRequirements);
					});
					//]]>
				</script>";
		}
		echo "
			<div id=\"minPasswordLength\" class=\"passwordReq small\">Password must be at least 9 characters (not yet met)</div>
			<div id=\"passwordDigitsRequired\" class=\"passwordReq small\">It must contain at least one digit (0-9) (not yet met)</div>
			<div id=\"passwordLowerCaseRequired\" class=\"passwordReq small\">It must contain at least one lower case letter (a-z) (not yet met)</div>
			<div id=\"passwordUpperCaseRequired\" class=\"passwordReq small\">It must contain at least one upper case letter (A-Z) (not yet met)</div>
			<div id=\"passwordSpecialRequired\" class=\"passwordReq small\">It must contain at least one special character (not yet met): <br />
				Valid special characters: <span style=\"font-family: Arial, Helvetica, sans-serif;\">~ ! @ # $ % ^ &amp; * ( ) _ + [ ] { } | ; , . &lt; &gt; ?</span></div>";
	}
	
	/**
	 * Returns a String representing what is wrong about the password, or null if nothing is wrong with it.
	 */
	function verifyPasswordStrength($input) {
		$numWrong = 0;
		$str = null;
		if (strlen($input) < MIN_LENGTH) {
			$numWrong++;
			$str = ($str == null ? "" : "$str<br />\n") . "The password must contain at least 9 characters.";
		}
		if (!preg_match('/[0-9]/', $input)) {
			$numWrong++;
			$str = ($str == null ? "" : "$str<br />\n") . "The password must contain at least one digit (0-9).";
		}
		if (!preg_match('/[a-z]/', $input)) {
			$numWrong++;
			$str = ($str == null ? "" : "$str<br />\n") . "The password must contain at least one lower case letter (a-z).";
		}
		if (!preg_match('/[A-Z]/', $input)) {
			$numWrong++;
			$str = ($str == null ? "" : "$str<br />\n") . "The password must contain at least one upper case letter (A-Z).";
		}
		if (!preg_match('/[' . SPECIAL . ']/', $input)) {
			$numWrong++;
			$str = ($str == null ? "" : "$str<br />\n") . "The password must contain at least one special character:<br />\n" .
					"Valid special characters: <span style=\"font-family: Arial, Helvetica, sans-serif;\">~ ! @ # $ % ^ & * ( ) _ + [ ] { } | ; , . < > ?</span>";
		}
		if ($numWrong > 1) {
			return "The password fails to meet several strength requirements:<br>\n$str";
		}
		return $str;
	}