<?
require_once('includes/portalutils.php');
require_once(GLOBAL_INCLUDES_DIR . 'washington_utils.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/SecureMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/employee.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/patient.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/fullnameformatter.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetMessageMessage.php');
//require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetProvidersSeenMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetLastProviderSeenMessage.php');

/**
 * Return the secure message with the given ID, and the UID of the recipient.  If $markRead is set, it determines
 * whether or not to immediately mark the message as read upon load.  If not set, true is assumed.
 */
function getSecureMessage($uid, $id, $markRead=true) {
	$msg = new GetMessageMessage('cec_messaging');
	$pubKeyData = getPublicKey(TRANSPORT_PUBLIC_KEY_FILE);
	$response = $msg->getMessage($uid, $id, $pubKeyData, $markRead);
	return $response;
}

/**
 * Returns an HTML string representing the reply and mark read/unread buttons, the subject, and the message.
 * The second parameter determines whether or not to include the "Pop-Out" button, which shows the message on
 * its own page with no headers, etc.
 */
function getSecureMessageHTML($message, $toUID, $includePopOut=false, $markRead=true, $patientDetails=null) {
	$id = $message->getID();
	//$toUID = $message->getFieldValue('toUID');
	if ($message != null) {
		$body = $message->getDecryptedText();
		$str = "";
		if ($body === false) {
			return "";
		} else {
			$headers = $message->getHeaders($body);
			$body = $message->removeHeaders($body);
			$str = "<div id=\"secureMessageWrapper\">";
			$str .= getButtonDiv($toUID, $id, $includePopOut, ($patientDetails == null));
			$str .= getSubjectDiv($message);
			$str .= getSenderOrRecipientDiv($message, "From", false);
			if ($patientDetails != null) {
				foreach ($patientDetails as $patField=>$patValue) {
					$str .= "<div><span>$patField: </span><span>$patValue</span></div>\n";
				}
			}
			$str .= getSentTimestampDiv($message);
			//I don't think height really needs to be restricted in any case.  Even really long messages look fine.
			//$str .= getMessageDiv($body, $includePopOut);
			//Assume text/plain unless another type is specified.
			$str .= getMessageDiv($body, $headers, false);
			$str .= "</div>";
			//If marking the message as read upon loading, do so in the GUI now.
			if ($markRead) {
				$str .= "<script type=\"text/javascript\">\n\tinnerMarkRead('$id', true);\n</script>";
			}
		}
		return $str;
	}
	return "";
}

/**
 * Returns HTML for a new message.  If $inReplyTo is specified, the new message will be initialized as a response
 * to the provided SecureMessage object.  For the effect of $inputPatientUID, see getSenderOrRecipientDiv()
 */
function getNewMessageHTML($inReplyTo=null, $replyToUID=null, $inputPatientUID=null, $providers=null, $availableProviders=null) {
	$editable = ($inReplyTo == null && $_SESSION['userMode'] == PATIENT_MODE);
	$str = "<form id=\"messageForm\" action=\"send_secure_message.php\" method=\"POST\"><div id=\"newMessageWrapper\">";
	$str .= "<div class=\"messageValidateError error\" style=\"display:none;\"></div>";
	$str .= getEditableSubjectDiv($inReplyTo);
	$str .= getEditableSenderDiv($inReplyTo, $availableProviders);
	$str .= getSenderOrRecipientDiv($inReplyTo, "To", $editable, $inputPatientUID, $providers, $availableProviders);
	$str .= getEditableMessageDiv($inReplyTo);
	$str .= getSendMessageDiv();
	$str .= "<div class=\"messageValidateError error\" style=\"display:none;\"></div>";
	if ($inReplyTo != null) {
		//$str .= "<input name=\"replyToRecipient\" type=\"hidden\" value=\"" . $inReplyTo->getFieldValue('toUID') . "\" />\n";
		$str .= "<input name=\"replyToRecipient\" type=\"hidden\" value=\"$replyToUID\" />\n";
		$str .= "<input name=\"replyToID\" type=\"hidden\" value=\"" . urlencode($inReplyTo->getID()) . "\" />\n";
	}
	$str .= "</div></form>
			<script type=\"text/javascript\">
				$('#messageForm').submit(function(event) {
					validateAndSendMessage(event);
				});
			</script>";
	return $str;
}

//Returns the div containing the "Reply" and "Mark Unread" buttons, as well as an optional "Pop-Out" button,
//for the message with the given ID.
function getButtonDiv($uid, $id, $includePopOut, $includePrintDetailed) {
	$onClickPrint = "";
	$popoutMessage = "";
	$printWithDetails = "";
	$userMode = $_SESSION['userMode'];
	if ($includePopOut) {
		$safeID = makeIDSafe("popOutForm_$id");
		$popoutMessage = "<a title=\"Show in a new tab\" href=\"javascript:popOut('$safeID')\"><img alt=\"Show in New Tab\" class=\"popOutImage\" src=\"/Imagess/pop_out.png\" /></a>";
		$popoutForm = "
			<form id=\"$safeID\" action=\"get_secure_message.php\" method=\"POST\" target=\"_blank\">
				<input type=\"hidden\" name=\"fullPage\" value=\"1\" />
				<input type=\"hidden\" name=\"id\" value=\"$id\" />
				<input type=\"hidden\" name=\"toUID\" value=\"$uid\" />
				<input type=\"hidden\" name=\"autoPrint\" id=\"" . $safeID . "_print\" value=\"0\" />
				<input type=\"hidden\" name=\"autoPrintDetailed\" id=\"" . $safeID . "_print_detailed\" value=\"0\" />
			</form>";
		$onClickPrint = "popOutPrint('$safeID', '" . $safeID . "_print')";
	} else {
		$onClickPrint = "window.print();";
	}
	if ($userMode == EMPLOYEE_MODE && $includePrintDetailed) {
		$printWithDetails = "<input type=\"button\" value=\"Print Detailed\" title=\"Print the message along with the patient's contact information\" onclick=\"popOutPrint('$safeID', '" . $safeID . "_print_detailed');\" />";
	}
	$escapedID = urlencode($id);
	$safeID = makeIDSafe($id);
	$str = "
		<div style=\"display: none;\">
			<form id=\"hiddenReplyForm_$safeID\" action=\"new_message.php?userMode=" . $_SESSION['userMode'] . "\" method=\"POST\">
				<input type=\"hidden\" name=\"replyTo\" value=\"$escapedID\" />
				<input type=\"hidden\" name=\"replyToUID\" value=\"$uid\" />
			</form>
		</div>
		<form><div id=\"messageButtonDiv\" class=\"messageSection\" valign=\"middle\">
			<input type=\"button\" value=\"Reply\" onclick=\"reply('hiddenReplyForm_$safeID')\" />
			<input type=\"button\" value=\"Mark Unread\" onclick=\"markRead('$uid', '$id', false)\" />
			<input type=\"button\" value=\"Print\" onclick=\"$onClickPrint\" />
			$printWithDetails
			$popoutMessage
		</div></form>$popoutForm";
	return $str;
}

//Returns the div containing the subject of the message
function getSubjectDiv($message) {
	$subject = $message->getFieldValue('subject');
	return "<h3 class=\"orange\">$subject</h3>";
}

//Returns the div containing the subject of the message
function getSentTimestampDiv($message) {
	$timestamp = $message->getSentTimestamp();
	return "<h4 class=\"smaller\">Sent on $timestamp</h4>";
}

//Returns the div containing the sender or recipient of the message
//$message - The SecureMessage object from which to pull data
//$field - The display text, either "To:" or "From:"
//$editable - Whether or not the field should be editable
//$inputPatientUID - The UID of the patient to whom the provider chose to send a message
//$availableProviders - Provider employees who have mailboxes
function getSenderOrRecipientDiv($message, $field, $editable, $inputPatientUID=null, $providers=null, $availableProviders=null) {
	$userMode = $_SESSION['userMode'];
	$str = "";
	if ($message != null) {
		$pat = new Patient(null);
		$pat->setFromFirstWhere('uid', $message->getFieldValue('fromUID'), '=', array($message->getBaseDB()));
		if ($pat->getFieldValue('uid') == null) {
			$pat = null;
			$emp = new Employee(null);
			$emp->setFromFirstWhere('uid', $message->getFieldValue('fromUID'), '=', array($message->getBaseDB()));
			if ($emp->getFieldValue('uid') != null) {
				$holder = $emp;
			}
		} else {
			$holder = $pat;
		}	
	} else if ($userMode == PATIENT_MODE) {
		$pat = getPatient(false);
	} else if ($userMode == EMPLOYEE_MODE) {
		//Get the patient from the input to this method
		$pat = new Patient(null);
		$pat->setFromFirstWhere('uid', $inputPatientUID, '=');
		if ($pat->getFieldValue('uid') == null) {
			$pat = null;
		}
		$holder = $pat;
	}
	$fnf = new FullNameFormatter(null, false);
	
	if (!$editable) {
		if (isset($holder)) {
			$name = ($pat == $holder ? "" : "Dr. ") . $fnf->format($holder);
			$holderUid = $holder->getFieldValue('uid');
			$str = "<div class=\"big messageSection\"><span>$field: </span><span>$name</span></div>\n";
			$str .= "<input type=\"hidden\" value=\"$holderUid\" name=\"messageRecipient\" id=\"messageRecipientHidden\" />\n";
		} else {
			$fieldAlt = ($field == 'To' ? "recipient" : "sender");
			$str = "<div class=\"bigorange\">$field: Could not find $fieldAlt.  Please try again.</div>";
		}
	} else if ($editable && $userMode == PATIENT_MODE) {
		//Patient mode - there must be a patient set
		if ($pat == null) {
			$str = "<div class=\"bigorange\">$field: Could not find available doctors.  Please try again.</div>";
		} else {
			$str = "<div id=\"providerSelectDiv\" class=\"bigorange messageSection\">$field: ";
			$str .= getProviderSelect($pat, $providers, $availableProviders) . "</div>";
		}
	} //End if editable and patient mode
	return $str;
}

/**
 * Returns a select for choosing a provider based on those the patient has seen.
 * $pat - The patient for whom to show the select of available providers
 * $providers - An array of Provider objects that represent those providers that have seen the patients
 * $availableProviders - An array of UID's of the providers with available mailboxes
 */
function getProviderSelect($pat, $providers=null, $availableProviders=null, $errorMessage=null) {
	if ($availableProviders == null) {
		$availableProviders = array();
	}
	
	$fnf = new FullNameFormatter(null, false);
	
	//Select the provider the patient saw last by default
	$msg = new GetLastProviderSeenMessage('cec_portal');
	try {
		$providerLastSeen = $msg->getLastProviderSeen($pat->getFieldValue('uid'));
	} catch (Exception $e) {
		//Log exceptions and move on
		klog("Error occurred while finding patient last provider seen\n" . dump($e), KLogger::ERROR);
	}
	
	//Find the providers that the patient has seen.  Include those providers as options in the list.
	//$msg = new GetProvidersSeenMessage('cec_portal');
	if ($errorMessage == null) {
		try {
			//$providers = $msg->getProvidersSeen($pat->getFieldValue('uid'));
			$str = "<select name=\"messageRecipient\" id=\"messageRecipientSelect\" required=\"required\">";
			$found = false;
			if ($providers != null && is_array($providers)) {
				foreach ($providers as $provider) {
					$proUID = $provider->getFieldValue('uid');
					if (in_array($proUID, $availableProviders)) {
						$found = true;
						$selected = "";
						if (isset($providerLastSeen) && $providerLastSeen->getFieldValue('uid') == $proUID) {
							$selected = " selected=\"selected\"";
						}
						$str .= "<option value=\"$proUID\"$selected>" . $fnf->format($provider) . "</option>";
					}
				}
			}
			if ($found) {
				$str .= "</select>";
			} else {
				if ($providers != null && is_array($providers) && count($providers) > 0) {
					//Patient has doctors, but they aren't set up yet.
					$str = "Your Doctor(s)' Accounts are not Ready<br /><br />
					<div class=\"basicMedium\">We are working now to finalize your doctors patient portal account.</div>
					<div class=\"basicMedium\">In the meantime you may send any questions or comments to <a class=\"basicLink\" href=\"mailto:info@webeca.com\">info@webeca.com.</a></div>
					<div class=\"basicMedium\">Your message will be passed along to your doctor.</div>
					<div class=\"basicMedium\">Thank you for your patience. Check back soon to begin communicating directly with your EyeCare Associates doctor.</div>";
				} else {
					//Patient hasn't been seen in EMR
					$str = "Your Doctors could not be Found<br /><br />
					<div class=\"basicMedium\">You do not appear to have been seen recently by any of our doctors.  You can only send secure messages to doctors with whom you have had an eye exam.  Please feel free to <a class=\"basicLink\" href=\"schedule_choose.php\">schedule an appointment</a> now!  If you have any questions, please send them to <a class=\"basicLink\" href=\"mailto:info@webeca.com\">info@webeca.com</a>.</div>";
				}
			}
		} catch (Exception $e) {
			$str = "Could not find available doctors.  Please try again.";
			klog("Error occurred while finding patient providers seen\n" . dump($e), KLogger::ERROR);
		}
	} else {
		$str = $errorMessage;
	}
	return $str;
}

//Returns the div containing the message itself
function getMessageDiv($body, $headers, $restrictHeight) {
	$restricted = ($restrictHeight ? "messageBodyRestricted " : "");
	$str = "<div id=\"messageBody\" class=\"" . $restricted . "messageSection\">";
	$contentType = 'text/plain';
	$transferEncoding = '';
	if ($headers != null && $headers['Content-Type'] != null) {
		$contentType = $headers['Content-Type'];
	}
	if ($headers != null && $headers['Content-Transfer-Encoding'] != null) {
		$transferEncoding = $headers['Content-Transfer-Encoding'];
	}
	
	if (stripos($transferEncoding, 'quoted-printable') !== false) {
		//If the transfer encoding is "quoted-printable", that means stuff is escaped with equals signs and each line
		//is limited to 76 characters.  Decode this before including the body.
		$body = quoted_printable_decode($body);
		//The &#160;'s are replaced with ASCII 0xA0.  Replace them back with a non-breaking space.
		$body = str_replace(chr(160), "&nbsp;", $body);
	}
	if (stripos($contentType, 'text/html') !== false) {
		$htmlBodyPos = stripos($body, "<body");
		if ($htmlBodyPos !== false) {
			$openEndPos = stripos($body, ">", $htmlBodyPos + 1);
			$htmlBodyEndPos = stripos($body, '</body>');
			if ($htmlBodyEndPos !== false) {
				$str .= substr($body, $openEndPos + 1, ($htmlBodyEndPos - $openEndPos) - 1);
			}
		}
	} else {
		$bodyLines = explode("\n", $body);
		$tabs = 0;
		$divs = 0;
		
		//Tab quoted text over with left borders like most email clients do, rather than just showing ">>>".
		foreach ($bodyLines as $bodyLine) {
			$lineTabs = 0;
			$withoutTabs = $bodyLine;
			while (strpos($withoutTabs, '&gt;') === 0 || strpos($withoutTabs, '>') === 0) {
				//Treat &gt; or > at the beginnings of lines as equivalent; I can't differentiate the two.
				if (strpos($withoutTabs, '>') === 0) {
					$withoutTabs = substr($withoutTabs, 1);
				} else {
					$withoutTabs = substr($withoutTabs, 4);
				}
				$lineTabs++;
			}
			while ($lineTabs > $tabs) {
				//We need extra divs
				$str .= "<div class=\"secureMessageQuoted\">";
				$tabs++;
			}
			while ($lineTabs < $tabs) {
				//We need extra divs
				$str .= "</div>";
				$tabs--;
			}
			$str .= $withoutTabs . "<br />\n";
		}
		//Close any necessary divs at the end
		for ($i = 0; $i < $tabs; $i++) {
			$str .= "</div>";
		}
	}
	
	$str .= "</div>";
	return $str;
}

/*
 * Returns the subject of a new message given the (optional) message to which it is a reply
 */
function getSubject($inReplyTo=null) {
	$subject = "";
	if ($inReplyTo != null) {
		//Get the subject from the original message
		$subject = $inReplyTo->getFieldValue('subject');
		if ($subject != null && stripos($subject, "re:") !== 0) {
			//Add "Re: " to the beginning if the message doesn't already start with "Re:"
			$subject = "Re: $subject";
		}
	}
	return $subject;
}

//Returns the div for viewing and editing the subject.  If the param is set, auto-populate the field with "Re: [original subject]"
function getEditableSubjectDiv($inReplyTo=null) {
	$subject = getSubject($inReplyTo);
	return "<div class=\"bigorange messageSection\">Subject: <input name=\"messageSubject\" id=\"messageSubjectField\" required=\"required\" type=\"text\" value=\"$subject\" style=\"color: #000000;\" /></div>";
}

//Returns the div for viewing and editing the message itself.  If the param is set, auto-populate the text area with
//the original text, but with ">" appended to the beginning of each line, and a couple of newlines at the beginning to
//show the user where to type.
function getEditableMessageDiv($inReplyTo=null) {
	//Create a text area for displaying the message.  If the original message is set, initialize the text area's text.
	$text = "";
	if ($inReplyTo != null) {
		//Decrypt the text.  If it could not be decrypted, just let the exception be thrown.
		$text = $inReplyTo->getDecryptedText();
		if ($text != null) {
			$headers = $inReplyTo->getHeaders($text);
			$contentType = 'text/plain';
			if ($headers != null && $headers['Content-Type'] != null) {
				$contentType = $headers['Content-Type'];
			}
			if (stripos($contentType, 'text/html') !== false) {
				//If the original message is HTML, do not include it in the response body
				$text = '';
			} else {
				$text = $inReplyTo->removeHeaders($text);
				if (strpos($text, "\r\n") !== false) {
					$textLines = explode("\r\n", $text);
				} else {
					$textLines = explode("\n", $text);
				}
				$alteredTextLines = array();
				foreach ($textLines as $textLine) {
					//Don't differentiate between '&gt;' and '>'
					if (strpos($textLine, ">") === 0 || strpos($textLine, "&gt;") === 0) {
						$alteredTextLines[] = "&gt;$textLine";
					} else {
						$alteredTextLines[] = "&gt; $textLine";
					}
				}
				$replyDate = $inReplyTo->getSentTimestamp();
				$pat = new Patient(null);
				$pat->setFromFirstWhere('uid', $inReplyTo->getFieldValue('fromUID'), '=', array($inReplyTo->getBaseDB()));
				if ($pat->getFieldValue('uid') == null) {
					$pat = null;
					$emp = new Employee(null);
					$emp->setFromFirstWhere('uid', $inReplyTo->getFieldValue('fromUID'), '=', array($inReplyTo->getBaseDB()));
					if ($emp->getFieldValue('uid') != null) {
						$holder = $emp;
					}
				} else {
					$holder = $pat;
				}
				$replyName = "<UNKNOWN SENDER>";
				if ($holder != null) {
					$fnf = new FullNameFormatter(null, false);
					$replyName = ($pat == $holder ? "" : "Dr. ") . $fnf->format($holder);
				}
				$replyHeader = "On $replyDate, $replyName wrote:";
				$text = "\n\n\n$replyHeader\n" . implode("\n", $alteredTextLines);
			}
		}
	}
	return "<textarea name=\"messageBody\" id=\"messageBodyArea\" class=\"messageSection\" required=\"required\">$text</textarea>";
}

//Returns the div that includes the "Discard" and "Send" buttons.
function getSendMessageDiv() {
	$patientMode = ($_SESSION['userMode'] == PATIENT_MODE ? "true" : "false");
	//Add the discard button.
	$str = "<div style=\"text-align: right;\"><input type=\"button\" value=\"Discard\" onclick=\"discardMessage($patientMode);\" />";
	//Add the send button
	$str .= "<input type=\"submit\" value=\"Send\" /></div>";
	return $str;
}

//Returns the div containing the sender, and potentially allows choosing the sender.
function getEditableSenderDiv($message, $availableProviders=null) {
	if ($availableProviders == null) {
		$availableProviders = array();
	}
	
	$fnf = new FullNameFormatter(null, false);
	$str = "";
	$options = null;
	if ($_SESSION['userMode'] == PATIENT_MODE) {
		if ($message != null) {
			$pat = new Patient(null);
			$pat->setFromFirstWhere('uid', $message->getFieldValue('toUID'), '=', array($message->getBaseDB()));
			if ($pat->getFieldValue('uid') != null) {
				$holder = $pat;
			}
		} else {
			//Not in reply to anything.  Allow all users of this account.
			$pat = getPatient(false);
			$options = ($pat == null ? null : getAllUnderAccount($pat));
			$holder = $pat;
		}
	} else {
		//Employee mode - use the logged in employee
		$emp = getEmployee(false);
		if ($emp->getFieldValue('uid') != null) {
			$holder = $emp;
		}
	}
	
	$editable = ($options != null && count($options) > 1);
	if ((!$editable) && isset($holder)) {
		$name = ($_SESSION['userMode'] == PATIENT_MODE ? "" : "Dr. ") . $fnf->format($holder);
		$holderUID = $holder->getFieldValue('uid');
		$str = "<div class=\"bigorange messageSection\"><span>From: </span><span>$name</span></div>\n";
		$str .= "<input type=\"hidden\" value=\"$holderUID\" name=\"messageSender\" id=\"messageSenderHidden\" />\n";
	} else if ($editable) {
		if ($pat == null) {
			$str = "<div class=\"bigorange\">From: COULD NOT FIND AVAILABLE PATIENTS</div>";
		} else {
			$str = "<div class=\"bigorange messageSection\">From: <select name=\"messageSender\">";
			//List each of the patient's children, and the patient themself.
			foreach ($options as $option) {
				$optionUID = $option->getFieldValue('uid');
				$optionName = $fnf->format($option);
				$str .= "<option value=\"$optionUID\">$optionName</option>";
			}
			$str .= "</select></div>
				<script type=\"text/javascript\">
					var providers = [";
			$first = true;
			foreach ($availableProviders as $availablePro) {
				if (!$first) {
					$str .= ", ";
				}
				$str .= "\"$availablePro\"";
				$first = false;
			}
			$str .=	"];
					$(document).ready(function() {
						$('select[name=\"messageSender\"]').change(function() {
							selectPatientProviders($(this).val(), 'providerSelectDiv', providers);
						});
					});
				</script>";
		}
	}
	return $str;
}

?>