<?
require_once('includes/portalutils.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/MessageSummary.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/provider.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/patient.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/fullnameformatter.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetMessagesMessage.php');

/**
 * Return a list of secure messages for the provided uid.  This method assumes that the user/patient with
 * the provided UID has been authenticated, and the method itself does not perform any user authentication.
 */
function getSecureMessages($uid) {
	//Use the secure message function to get 
	$msg = new GetMessagesMessage('cec_messaging');
	$response = $msg->getMessages($uid);
	return $response;
}

/**
 * Returns an HTML list of all of the provided messages, as check boxes so they can be individually
 * selected/deselected for various purposes.
 */
function getSecureMessagesHTML($uid, $messages, $messageToShow=null, $showFirst=false) {
	//$messages = getSecureMessages($uid);
	$str = "<form><div id=\"mainMessagesDiv\"><div id=\"innerMessagesDiv\">";
	
	//Add "Mark Read" and "Mark Unread" buttons
	//TODO - ADD A "Compose" BUTTON
	$composeText = "";
	if ($_SESSION['userMode'] == PATIENT_MODE) {
		//Only add a compose button in patient mode.
		$composeText = "<input type=\"button\" onclick=\"compose(" . $_SESSION['userMode'] . ")\" value=\"Compose\" />";
	}
	$str .= "
		<div id=\"messagesButtonDiv\">
			$composeText
			<span style=\"margin-left: 20px;\">With selected: </span>
			<input type=\"button\" onclick=\"markSelectedRead('$uid', true)\" value=\"Mark as Read\" />
			<input type=\"button\" onclick=\"markSelectedRead('$uid', false)\" value=\"Mark as Unread\" />
		</div> <!-- End div: buttonDiv -->
		<div id=\"messagesTableOuterDiv\"><table id=\"messagesTable\">
			<tr class=\"messagesHeaderRow\">
				<th style=\"width: 20px;\"><img class=\"clickable\" onclick=\"selectAllMessages()\" src=\"/Imagess/check.png\" alt=\"Select All\" /></th>
				<th style=\"width: 25%;\">Sent</th>
				<th style=\"width: 25%;\">Sender</th>
				<th style=\"width: 45%;\">Subject</th>
			</tr>";
	$fnf = new FullNameFormatter(null, false);
	$messageDivID = "mainMessageDiv";
	$errorDivID = "messageErrorDiv";
	$scriptStr = null;
	$toUID = null;
	if ($messages != null) {
		foreach ($messages as $message) {
			$messageID = $message->getId();
			$toUID = $message->getFieldValue('toUID');
			$messageSubject = $message->getSubject();
			$messageOther = $message->getOtherEntity($uid);
			$messageClass = ($message->isRead() ? "read" : "unread");
			if ($messageOther != null && $messageOther != "") {
				//First check to see if there's an employee with the given UID
				$employee = new Employee(null);
				if ($employee->setFromFirstWhere('uid', $messageOther, '=')) {
					$sender = $fnf->format($employee);
				} else {
					//No employee with that UID.  Next check to see if there's a patient with the given UID.
					$patient = new Patient(null);
					if ($patient->setFromFirstWhere('uid', $messageOther, '=')) {
						$sender = $fnf->format($patient);
					}
				}
			}
			$sentDate = $message->getSentTimestamp('m/d/Y \\a\\t h:i:s A');
			
			//Add an identifiable check box, the message sender/recipient, and subject.
			$messageIDEscaped = makeIDSafe("_" . $messageID);
			$str .= "
				<tr>
					<td style=\"width: 20px;\" class=\"checkCell\"><input type=\"checkbox\" class=\"messageCheck\" message_id=\"$messageID\" /></td>
					<td id=\"sent$messageIDEscaped\" onclick=\"selectMessage('$messageDivID', '$errorDivID', '$messageID', '$toUID');\" class=\"$messageClass\" style=\"overflow: hidden; width: 25%;\" ><div class=\"messageSummaryDiv\" title=\"$sentDate\">$sentDate</div></td>
					<td id=\"sender$messageIDEscaped\" onclick=\"selectMessage('$messageDivID', '$errorDivID', '$messageID', '$toUID');\" class=\"$messageClass\" style=\"overflow: hidden; width: 25%;\" ><div class=\"messageSummaryDiv\" title=\"$sender\">$sender</div></td>
					<td id=\"subject$messageIDEscaped\" onclick=\"selectMessage('$messageDivID', '$errorDivID', '$messageID', '$toUID');\" class=\"$messageClass\" style=\"overflow: hidden; width: 45%;\" ><div class=\"messageSummaryDiv\" title=\"$messageSubject\">$messageSubject</div></td>
				</tr>";
			
			if ($messageToShow != null && $scriptStr == null && $messageID == $messageToShow) {
				$scriptStr = "
					<script type=\"text/javascript\">
						$(window).load(function() {
							selectMessage('$messageDivID', '$errorDivID', '$messageID', '$toUID');
						});
					</script>";
			} else if ($showFirst && $scriptStr == null) {
				$scriptStr = "
					<script type=\"text/javascript\">
						$(window).load(function() {
							selectMessage('$messageDivID', '$errorDivID', '$messageID', '$toUID');
						});
					</script>";
			}
		}
	}
	$scriptStr = ($scriptStr == null ? "" : $scriptStr);
	$str .= "
		</table></div> <!-- End div: innerMessagesDiv --> </div> <!-- End div: messagesTableOuterDiv -->
		<div id=\"$messageDivID\" style=\"display: none;\"></div> <!-- End div: $messageDivID -->
		<div id=\"$errorDivID\" class=\"error\" style=\"display: none;\"></div> <!-- End div: $errorDivID -->
		</div> <!-- End div: mainMessagesDiv --> </form>$scriptStr";
	return $str;
}

?>