<?

require_once(GLOBAL_INCLUDES_DIR . 'springfield/invalidelement.php');
require_once('get_receipt_html.php');

//****************************************************************************************************************
//***** Call this function to generate a credit card bill and save the superbill.                            *****
//******* Assumes the superbill has already been validated                                                   *****
//***** Params:                                                                                              *****
//******* $patient - REQUIRED! The Patient for the superbill                                                 *****
//******* $superbill - REQUIRED! The Superbill object                                                        *****
//******* $creditcard - REQUIRED! The CreditCard object to bill                                              *****
//******* $cc_amount - REQUIRED! The amount to bill to the credit card                                       *****
//******* $tax - REQUIRED! The amount of tax (it's also part of cc_amount)                                   *****
//******* $shippingCost - The amount of shipping (it's also part of cc_amount, default $0.00)                *****
//******* $testMode - true for testing (fake cards, no billing), false for live credit cards (default false) *****
//******* $dontSubmit - controls whether or not the superbill is actually submitted (default false)          *****
//******* $email - email address to send the receipt to (default null)                                      *****
//***** Returns:                                                                                             *****
//******* null upon success                                                                                  *****
//******* Set of InvalidElements if                                                                          *****
//****************************************************************************************************************
function submitSuperbillPayment($patient, $superbill, $creditcard, $cc_amount, $tax,
                                $shippingCost=0, $testMode=false, $dontSubmit=false, $email=null) {
	$patientFieldArray = $patient->getFields(array('firstname', 'lastname', 'address', 'city', 'state', 'zip', 'homephone'));

	//Set the billing info defaults array based on either previously set billing info or the patient's array
	if (isset($_SESSION['billto'])) {
		$billingArray = $_SESSION['billto'];
	} else {
		$billingArray = $patientFieldArray;
	}

	//$email = $patient->getFieldValue('email');
	$superbill_number = $superbill->getSuperbillNumber();
	$db = $superbill->getDatabaseName();
	$company = (($db == 'cec') ? "CEC" : "Vision Corp");
	$dbs = unserialize(SERVERS);
	$apiID = $dbs[$db]['api_id'];
	$transactionKey = $dbs[$db]['transaction_key'];
	
	//This is if we use a "test account"... I guess in case our authorize.net account expires or something
	//$auth_net_url				= "https://test.authorize.net/gateway/transact.dll";
	//$ch = curl_init("https://test.authorize.net/gateway/transact.dll"); 

	//This is for using our live authorize.net account
	$auth_net_url = "https://secure.authorize.net/gateway/transact.dll";
	$ch = curl_init("https://secure.authorize.net/gateway/transact.dll");

	//If explicitly testing or if not submitting superbill (don't want to accidentally bill someone w/o submitting a superbill!)
	if ($testMode || $dontSubmit) {
		$authnet_values = array(
			"x_login"				=> $apiID,
			"x_version"				=> "3.1",
			"x_delim_char"			=> "|",
			"x_delim_data"			=> "TRUE",
			"x_url"					=> "FALSE",
			"x_type"				=> "AUTH_CAPTURE",
			"x_method"				=> "CC",
			"x_tran_key"			=> $transactionKey,
			"x_relay_response"		=> "FALSE",
			"x_duplicate_window"    => "60",
			
			"x_card_num"			=> TEST_VISA,  //This is only a test
			"x_exp_date"			=> TEST_EXP,   //This is only a test
			"x_description"			=> "Webeca.com $company Online Transaction",
			"x_amount"				=> number_format($cc_amount, 2),
			"x_tax" 				=> number_format($tax, 2),
			"x_freight"				=> $shippingCost,
			
			"x_first_name"			=> $billingArray['firstname'],
			"x_last_name"			=> $billingArray['lastname'],
			
			"x_address"				=> $billingArray['address'],
			"x_city"				=> $billingArray['city'],
			"x_state"				=> $billingArray['state'],
			"x_zip"					=> $billingArray['zip'],
			
			"x_phone"				=> $billingArray['homephone'],
			
			"x_customer_ip"			=> $_SERVER['REMOTE_ADDR'], 
			
			"x_email"				=> $email,
			"x_email_customer"      => "FALSE", 
			
			"x_invoice_num"			=> $superbill_number,
			
			"x_ship_to_first_name"			=> $patientFieldArray['firstname'],
			"x_ship_to_last_name"			=> $patientFieldArray['lastname'],
			
			"x_ship_to_address"				=> $patientFieldArray['address'],
			"x_ship_to_city"				=> $patientFieldArray['city'],
			"x_ship_to_state"				=> $patientFieldArray['state'],
			"x_ship_to_zip"					=> $patientFieldArray['zip'],
			"x_ship_to_phone"				=> $patientFieldArray['homephone'],
			
			"x_test_request"				=> "true" //This is only a test
		);
	} else {
		$authnet_values = array(
			"x_login"				=> $apiID,
			"x_version"				=> "3.1",
			"x_delim_char"			=> "|",
			"x_delim_data"			=> "TRUE",
			"x_url"					=> "FALSE",
			"x_type"				=> "AUTH_CAPTURE",
			"x_method"				=> "CC",
			"x_tran_key"			=> $transactionKey,
			"x_relay_response"		=> "FALSE",
			"x_duplicate_window"    => "60",
			
			"x_card_num"			=> $creditcard->getNumber(), 
			"x_exp_date"			=> $creditcard->getExpirationString(),
			"x_description"			=> "Webeca.com.com $company Online Transaction",
			"x_amount"				=> number_format($cc_amount, 2),
			"x_tax" 				=> number_format($tax, 2),
			"x_freight"				=> $shippingCost,
			
			"x_first_name"			=> $billingArray['firstname'],
			"x_last_name"			=> $billingArray['lastname'],
			
			"x_address"				=> $billingArray['address'],
			"x_city"				=> $billingArray['city'],
			"x_state"				=> $billingArray['state'],
			"x_zip"					=> $billingArray['zip'],
			
			"x_phone"				=> $billingArray['homephone'],
			
			"x_customer_ip"			=> $_SERVER['REMOTE_ADDR'], 
			
			"x_email"				=> $email,
			"x_email_customer"      => "FALSE", 
			
			"x_invoice_num"			=> $superbill_number,
			
			"x_ship_to_first_name"			=> $patientFieldArray['firstname'],
			"x_ship_to_last_name"			=> $patientFieldArray['lastname'],
			
			"x_ship_to_address"				=> $patientFieldArray['address'],
			"x_ship_to_city"				=> $patientFieldArray['city'],
			"x_ship_to_state"				=> $patientFieldArray['state'],
			"x_ship_to_zip"					=> $patientFieldArray['zip'],
			"x_ship_to_phone"				=> $patientFieldArray['homephone'],
		);
	}
	$fields = "";
	foreach( $authnet_values as $key => $value ) $fields .= "$key=" . urlencode( $value ) . "&";

	curl_setopt($ch, CURLOPT_HEADER, 0); // set to 0 to eliminate header info from response
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // Returns response data instead of TRUE(1)
	curl_setopt($ch, CURLOPT_POSTFIELDS, rtrim( $fields, "& " )); // use HTTP POST to send form data
	if (defined('CACERT_FILE')) {
		curl_setopt($ch, CURLOPT_CAINFO, CACERT_FILE); //Use this CA cert file to verify peer instead of curl default  
	}
	// curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE); // uncomment this line if you get no gateway response.  It means we need to update the CA cert file.  Recomment when we do get a valid response after updating CA cert file ###
	$resp = curl_exec($ch); //execute post and get results
	curl_close ($ch);

	$responseArray = explode('|', $resp);
	
	if ($responseArray[0] == 1) {
		// a code of 1 means it submitted without error.
		if ($dontSubmit || $superbill->updateSuperbill()) {
			//Finish the process - either we aren't submitting the superbill or it submitted successfully
			if (isset($email)) {
				$subject = "Webeca.com.com Invoice Receipt";
				$message = getReceiptHTML($superbill, $creditcard, $billingArray, $cc_amount);
				$headers = unserialize(RECEIPT_MAIL_HEADER_ARRAY);
				$headers['Subject'] = $subject;
				pearMail(SMTP_HOST, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD, $email, $message, $headers);
			}
			return null;
		} else {
			//If there was an error in that part, send an error email
			$subject = "URGENT: Webeca.com.com Superbill Error";
			//$message = "WARNING: Receiving this message means that someone was charged without the superbill being updated!\n\n";
			//$message .= "There was an error while saving superbill " . $superbill_number . " to the CMS.\n\n";
			//$message .= "Patient Information:\n";
			$errorID = date('YmdHisu');
			$message = "WARNING: Receiving this message means that someone was charged without the superbill being updated!\n\n";
			$message .= "There was an error while saving superbill " . $superbill_number . " to the CMS, Error ID $errorID.\n\n";
			
			$log = "URGENT ERROR: Superbill error, ID = $errorID\n";
			$log .= "Receiving this message means that someone was charged without the superbill being updated!\n\n";
			$log .= "Patient Information:\n";
			$patientFieldArray['city'] = str_replace('&amp;', '&', $patientFieldArray['city']);
			
			//$message .= $patientFieldArray['firstname'].' '.$patientFieldArray['lastname']."\n";
			//$message .= $patientFieldArray['address']."\n";
			//$message .= $patientFieldArray['city'].', '.$patientFieldArray['state'].' '.$patientFieldArray['zip']."\n";
			//$message .= 'Email Address: '.$email."\n\n";
			//$message .= 'Purchased On: '.date('m/d/Y')."\n\n";
			//$message .= "Authorize.net values:\n";
			$log .= $patientFieldArray['firstname'].' '.$patientFieldArray['lastname']."\n";
			$log .= $patientFieldArray['address']."\n";
			$log .= $patientFieldArray['city'].', '.$patientFieldArray['state'].' '.$patientFieldArray['zip']."\n";
			$log .= 'Email Address: '.$email."\n\n";
			$log .= 'Purchased On: '.date('m/d/Y')."\n\n";
			$log .= "Authorize.net values:\n";
			$authnet_values['x_card_num'] = "xxxx"; //don't show credit card num
			$authnet_values['x_login'] = "xxxx"; //don't show login
			$authnet_values['x_tran_key'] = "xxxx"; //don't show transaction key
			
			//$message .= dump($authnet_values);
			$log .= dump($authnet_values);
			klog($log, KLogger::ERROR);
			
			$headers = unserialize(ERROR_MAIL_HEADER_ARRAY);
			$headers['Subject'] = $subject;
			pearMail(SMTP_HOST, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD, ERROR_MAIL_TO, $message, $headers);
			$_SESSION['error'] = 'superbill';
			return new InvalidElement('superbill', 'superbill');
		}
	} else {
		//if it didn't, it failed for whatever reason is in $responseArray[3]
		//$_SESSION['error']['code'] = $responseArray[0];
		//$_SESSION['error']['string'] = $responseArray[3];
		$subject = "Webeca.com.com Credit Card Billing Error";
		$authnet_values['x_card_num'] = "xxxx"; //don't show credit card num
		$authnet_values['x_login'] = "xxxx"; //don't show login
		$authnet_values['x_tran_key'] = "xxxx"; //don't show transaction key
		//$message = "There was an authorize.net error in authorizing or capturing the billing.\n\n";
		//$message .= "ERROR CODE: " . $responseArray[0] . "\nERROR STRING: " . $responseArray[3] . "\n\nPARAMETERS: " . dump($authnet_values);
		//$message .= "\n\nOVERALL RESPONSE: " . dump($responseArray);
		$errorID = date('YmdHisu');
		$log = "Error: ID=$errorID \nThere was an authorize.net error in authorizing or capturing the billing.\n\n";
		$log .= "ERROR CODE: " . $responseArray[0] . "\nERROR STRING: " . $responseArray[3] . "\n\nPARAMETERS: " . dump($authnet_values);
		$log .= "\n\nOVERALL RESPONSE: " . dump($responseArray);
		
		//Blank out the credit card on the response array as well
		if (count($responseArray) > 51) {
			$responseArray[50] = 'XXXX';
			$responseArray[51] = '';
		}
		$message = "Billing Error, ID = $errorID, superbill = $superbill_number\nauthorize.net response = " . $responseArray[3];
		
		klog($log, KLogger::ERROR);
		
		$headers = unserialize(ERROR_MAIL_HEADER_ARRAY);
		$headers['Subject'] = $subject;
		pearMail(SMTP_HOST, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD, ERROR_MAIL_TO, $message, $headers);
		$_SESSION['error'] = $responseArray[3];
		return new InvalidElement($responseArray[0], $responseArray[3]);
	}
} //end function
?>
