<?
require_once('includes/conf.php');
enforcedSessionStart();

require_once('includes/portalutils.php');
require_once('includes/ZipStream/zipstream.php');
require_once(GLOBAL_INCLUDES_DIR . 'process_xslt.php');
require_once(GLOBAL_INCLUDES_DIR . 'config.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/AuditRecord.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/invalidelement.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/fullnameformatter.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/CreatePatientSummaryMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/SendPatientSummaryMessage.php');

function loadDirectAddresses($fileName) {
	$handle = @fopen($fileName, "r");
	$addresses = array();
	try {
		if ($handle) {
			//If the file can be found and loaded, load its contents (else just use empty array)
			while (($buffer = fgets($handle)) !== false) {
				$buffer = trim($buffer);
				if ($buffer != '') {
					$addresses[] = $buffer;
				}
			}
			if (!feof($handle)) {
				klog("Unexpected failure reading direct addresses", KLogger::ERROR);
			}
		}
	} catch (Exception $e) {
		//Log exceptions and move on
		klog("Error loading direct addresses:\n" . dump($e), KLogger::ERROR);
	}
	//I want this in a finally block, but they don't exist until PHP 5.5!
	fclose($handle);
	asort($addresses);
	return $addresses;
}

function addDirectAddress($fileName, $address, $newline = true) {
	$handle = @fopen($fileName, "a");
	$addresses = array();
	try {
		if ($handle) {
			//If the file can be found and loaded, load its contents (else just use empty array)
			if ($newline) {
				fwrite($handle, "\n");
			}
			fwrite($handle, $address);
		}
	} catch (Exception $e) {
		//Log exceptions and move on
		klog("Error loading direct addresses:\n" . dump($e), KLogger::ERROR);
		fclose($handle);
		return false;
	}
	//I want this in a finally block, but they don't exist until PHP 5.5!
	fclose($handle);
	return true;
}

function getCDA($patient, $portalUser, $method) {
	$msg = new CreatePatientSummaryMessage('cec_portal');
	$summary = $msg->createPatientSummary($patient->getFieldValue('uid'), $portalUser->getFieldValue('uid'), $method);
	return $summary;
}

//Make sure patient is logged in first
$mainPatient = getPatient();
$logged_in = isLoggedIn($mainPatient);
$patientNum = $_GET['patientNum'];
$loadPage = true;
$fnf = new FullNameFormatter(null, false);
$directAddresses = loadDirectAddresses(DIRECT_ADDRESSES_FILE);

//Find the patient that has been chosen
$chosenPatient = null;
if (!isset($patientNum)) {
	$patientNum = 0;
	$chosenPatient = $mainPatient;
}
$patients = getAllUnderAccount($mainPatient);
$curNum = 0;
foreach ($patients as $patient) {
	if ($curNum == $patientNum) {
		$chosenPatient = $patient;
		break;
	}
	$curNum++;
}
$error = null;
if (isset($_POST['downloadCDA']) && $_POST['downloadCDA'] == 1) {
	$options = array('send_http_headers'=>true);
	if (isset($chosenPatient)) {
		try {
			$cdaDoc = getCDA($chosenPatient, $mainPatient, AuditRecord::$DOWNLOAD_EXPORT_METHOD);
		} catch (Exception $e) {
			//The CDA doc will just be null, which triggers the error message below.
		}
		if (isset($cdaDoc)) {
			ob_clean();
			$zip = new ZipStream(SHORT_COMPANY_NAME . 'HealthInformation.zip', $options);
			$zip->add_file('HealthInformationRaw.xml', $cdaDoc->saveXML());
			$zip->add_file('HealthInformationReadable.html', processXSLT(getDOMFromFile(CCDA_STYLE_DOC_LOCATION), $cdaDoc));
			$zip->finish();
			
			$loadPage = false;
		} else {
			$error = "Error downloading health information.  Please try again later.";
		}
	} else {
		$error = "Error downloading health information.  Please try again later.";
	}
} else if (isset($_POST['transmitCDA']) && $_POST['transmitCDA'] == 1) {
	$transmitAddress = $_POST['transmitAddress'];
	$knownAddress = $_POST['knownAddress'];
	$transmitSubject = $_POST['transmitSubject'];
	if (($transmitAddress == null || $transmitAddress == '') && ($knownAddress == null || $knownAddress == '')) {
		$error = "You must choose an address to which to transmit the health summary, either from the list of known addresses or as a custom address entered with free text.";
	} else if ($transmitSubject == null) {
		$error = "You must choose a subject for the transmission message.";
	} else {
		$transmitAddress = (($transmitAddress == null || $transmitAddress == '') ? $knownAddress : $transmitAddress);
		$msg = new SendPatientSummaryMessage('cec_direct');
		try {
			$msgID = $msg->sendPatientSummary($chosenPatient->getFieldValue('uid'), $mainPatient->getFieldValue('uid'), $transmitAddress, $transmitSubject);
			$successMsg = "Health summary successfully submitted to $transmitAddress.";
			//Sent successfully - save the direct address recipient for future use
			if ($transmitAddress != null && $transmitAddress != '' && $transmitAddress != $knownAddress && !in_array($transmitAddress, $directAddresses)) {
				//User didn't use a known address, or changed it in some way.
				if (addDirectAddress(DIRECT_ADDRESSES_FILE, $transmitAddress, (count($directAddresses) > 0))) {
					$directAddresses[] = $transmitAddress;
				}
			}
		} catch (Exception $e) {
			$error = "Error transmitting message to $transmitAddress";
		}
	}
}

if ($loadPage) {
	if (!$logged_in) {
		$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
		header('location:login.php');
	} else {
		//Include the header info
		$title='EyeCare Associates - View, Download, and Transmit Health Information';
		include("template.php");
		
		$drawArrow = false;
		$subMenu = "portal_menu.jpg";
		$rightPadding = 50;
		include("head.php");
		
		echo "<h1 class=\"orange\">View, Download, and Transmit Health Information</h1>\n";
		
		if (isset($error)) {
			echo "<div class=\"error\">$error</div>";
		} else if (isset($successMsg)) {
			echo "<h2 class=\"orange\">Success!</h2><div class=\"basicMedium boldText\">$successMsg</div>";
		}
		
		//Loop through each patient under the account
		$num = 0;
		
		//Only list patients under the account if > 1
		if (isset($patients) && count($patients) > 1) {
			echo "<h2 class=\"orange\">Choose Patient</h2>\n<div class=\"spaced\">";
			foreach ($patients as $patient) {
				echo "<div class=\"basicMedium boldText\">";
				if ($num == $patientNum) {
					//This is the chosen patient; just show their name
					echo "-&gt;" . $fnf->format($patient);
				} else {
					//This is another patient, allow user to choose them
					echo "<a class=\"basicLink boldLink\" href=\"" . $_SERVER['PHP_SELF'] . "?patientNum=$num\">" . $fnf->format($patient) . "</a>";
				}
				echo "</div>\n";
				$num++;
			}
			echo "</div><br />\n";
		}
		
		if (isset($chosenPatient)) {
			//Load the patient's CCR (don't reload if it was already loaded above)
			$exceptionOccurred = false;
			if (!isset($cdaDoc)) {
				try {
					$cdaDoc = getCDA($chosenPatient, $mainPatient, AuditRecord::$VIEW_EXPORT_METHOD);
				} catch (Exception $e) {
					$exceptionOccurred = true;
				}
			}
			//Show the C-CDA as stylized by the xsl document
			if (isset($cdaDoc)) {
			?>

				<h2 class="orange">Choose an option below to view, download or transmit your health information to a third party</h2>
				<p>Your health information can be viewed below.  You may choose to either download or transmit this information in this section.</p>
<p>&nbsp;</p>
<div id="box-wrapper">
<div id="left-box-wrapper">
<div id="vdt-small-box"><h3 class="orange">View Health Information</h3>
<p>Click <a href="#summary">here</a> to view a comprehensive summary of your health information</p>
	</div>
                    <div id="spacer"></div>
<div id="vdt-small-box"><h3 class="orange"> Download</h3>
		<p>Click on the "Download" button to download your health information.</p>
				<form id="downloadForm" method="post" action="view_chart.php?patientNum=<? echo $patientNum; ?>">
					<input type="hidden" name="downloadCDA" value="1" />
					<input type="hidden" id="cdaSaveData" name="cdaSaveData" value="" />
					<input type="submit" value="Download" />
				</form></div>
                
                <div id="vdt-activitylog"><h3 class="orange">View Activity Log</h3>
				<div id="activityLogWrapper">
					<div id="activityLogTemp">
						<p>Click the "View Activity Log" button below to view the history of when you or an authorized representative has viewed, downloaded, or transmitted your health information.</p>
						<button type="button" onclick="viewActivityLog()">View Activity Log</button>
					</div>
                    </div>
                    </div>
                
                
    <div id="spacer"></div>
                

                    
                    
                    
  </div>
<div id="vdt-box"><h3 class="orange">Transmit Your Health Information to a Third Party</h3>
		<p>In order to securely transmit your health information to another medical practice we require a secure Direct Messaging Address. Please contact your doctor's office and request this Direct Messaging Address, then enter that address as well as a subject line below.</p>
				<br/><p>Please note that if our system cannot verify that address as secure your information will not be sent.</p><form method="post" action="view_chart.php?patientNum=<? echo $patientNum; ?>">
					<input type="hidden" name="transmitCDA" value="1" />
					<?
					if ($directAddresses != null && count($directAddresses) > 0) {
						echo "<div class=\"spacedMore\">\n";
						echo "<label for=\"knownAddress\">Select A Previously Used Email Address:<br/> </label>\n";
						echo "<select id=\"knownAddress\">\n";
						echo "<option value=\"\"> </option>\n";
						foreach ($directAddresses as $knownAddress) {
							echo "<option value=\"" . htmlspecialchars($knownAddress, ENT_QUOTES) . "\">$knownAddress</option>\n";
						}
						echo "</select>\n";
						echo "</div>";
						echo "<div class=\"spacedMore\">OR</div>";
					}
					?>
					<div class="spacedMoreTop">
						<label for="transmitAddress">Direct clinician's address to which to transmit:<br /> (*Not a standard email address.  Please read the paragraph above.)<br /> </label><input id="transmitAddress" name="transmitAddress" type="text" size="40" maxlength="255" <? echo ($messageAddress == null ? "" : "value=\"$messageAddress\" ") ?>/>
					</div>
					<div class="spacedMore">
						<label for="transmitSubject">Message subject: <br /></label><input id="transmitSubject" name="transmitSubject" type="text" size="40" maxlength="255" <? echo ($messageSubject == null ? "" : "value=\"$messageSubject\" ") ?>/>
					</div>
					<div>
						<input type="submit" value="Transmit" />
					</div>
				</form>
</h3></div><div class="spacer"></div>
</div>
                
                
 <div id="summary-wrapper">               
               
							
	            <a name="summary" id="summary"></a>	            <script type="text/javascript">
					var patient = '<? echo $chosenPatient->getFieldValue('uid'); ?>';
					
					function viewActivityLog() {
						//An exam is chosen.  Load the clinical summary.
						var args = { patUID: patient, from: '1970-01-01', to: '<? echo date("Y-m-d", add_date(time(), 0, 0, 1)); ?>' };
						var h4 = document.createElement("h4");
						h4.id = "loadingHistoryHeader";
						h4.appendChild(document.createTextNode("Loading history.  Please wait..."));
						$("#activityLogWrapper").append(h4);
						$("#activityLogTemp").hide();
						//$("#activityLogWrapper").html('<h4>Loading history.  Please wait...</h4>');
						$.post("get_patient_activity.php", args, function(data) {
							$("#loadingHistoryHeader").hide();
							$("#activityLogWrapper").append(data);
						});
					}
					
					function useKnownAddress(event) {
						$("#transmitAddress").val($("#knownAddress").val());
					}
					
					$(document).on('change', '#knownAddress', {}, useKnownAddress);
				</script>
			<?
				
				echo processXSLT(getDOMFromFile(CCDA_STYLE_DOC_LOCATION), $cdaDoc);
			} else {
				echo "<div class=\"bigorange spaced\">Health information not found for " . $fnf->format($chosenPatient) . "</div>\n";
				echo "<div class=\"smallItem\">Your health information could not be found at this time. Please try again later.</div>";
			}
			echo "\n";
		} else if ($exceptionOccurred) {
			echo "<div class=\"bigorange spaced\">Health information could not be loaded</div>\n";
			echo "<div class=\"smallItem\">We're sorry, but your health information could not be loaded at this time. Please try again later.</div>";
		} else {
			echo "<div class=\"bigorange spaced\">Health information not found</div>\n";
			echo "<div class=\"smallItem\">Your health information could not be found at this time. Please try again later.</div>";
		}
		
	} //end if logged in
} //End if load page
?></div>

<?  include "foot.php"; ?>