<?
require_once('includes/conf.php');
enforcedSessionStart();

require_once('includes/portalutils.php');
require_once('includes/ZipStream/zipstream.php');
require_once(GLOBAL_INCLUDES_DIR . 'process_xslt.php');
require_once(GLOBAL_INCLUDES_DIR . 'config.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/AuditRecord.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/invalidelement.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/dataholderfieldcomparator.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/formatters/fullnameformatter.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/CreateDefaultClinicalSummaryMessage.php');
require_once(GLOBAL_INCLUDES_DIR . 'springfield/backend/axis/GetPatientExamsMessage.php');

function getCDA($examUID, $portalUser, $method) {
	$msg = new CreateDefaultClinicalSummaryMessage('cec_portal');
	$summary = $msg->createDefaultClinicalSummary($examUID, $portalUser->getFieldValue('uid'), $method);
	return $summary;
}

//Make sure patient is logged in first
$mainPatient = getPatient();
$logged_in = isLoggedIn($mainPatient);
$patientNum = $_GET['patientNum'];
$loadPage = true;
$fnf = new FullNameFormatter(null, false);
//Find the patient that has been chosen
$chosenPatient = null;
if (!isset($patientNum)) {
	$patientNum = 0;
	$chosenPatient = $mainPatient;
}
$patients = getAllUnderAccount($mainPatient);
$curNum = 0;
foreach ($patients as $patient) {
	if ($curNum == $patientNum) {
		$chosenPatient = $patient;
		break;
	}
	$curNum++;
}
$error = null;
if (isset($_POST['downloadCDA']) && $_POST['downloadCDA'] == 1) {
	$options = array('send_http_headers'=>true);
	if (isset($_POST['examUID'])) {
		try {
			$cdaDoc = getCDA($_POST['examUID'], $mainPatient, AuditRecord::$DOWNLOAD_EXPORT_METHOD);
		} catch (Exception $e) {
			//The CDA doc will just be null, which triggers the error message below.
		}
		if (isset($cdaDoc)) {
			ob_clean();
			$zip = new ZipStream(SHORT_COMPANY_NAME . 'HealthInformation.zip', $options);
			$zip->add_file('ClinicalSummaryRaw.xml', $cdaDoc->saveXML());
			$zip->add_file('ClinicalSummaryReadable.html', processXSLT(getDOMFromFile(CCDA_STYLE_DOC_LOCATION), $cdaDoc));
			$zip->finish();
			$loadPage = false;
		} else {
			$error = "Error downloading clinical summary.  Please try again later.";
		}
	} else {
		$error = "Error downloading clinical summary.  Please try again later.";
	}
}

if ($loadPage) {
	if (!$logged_in) {
		$_SESSION['referral_page'] = $_SERVER['PHP_SELF'];
		header('location:login.php');
	} else {
		//Include the header info
		$title='EyeCare Associates - View Clinical Summary';
		include("template.php");
		
		$drawArrow = false;
		$subMenu = "portal_menu.jpg";
		$rightPadding = 50;
		include("head.php");
		
		echo "<h1 class=\"orange\">View a Clinical Summary of a Single Visit</h1>\n";
		
		if (isset($error)) {
			echo "<div class=\"error\">$error</div>";
		} else if (isset($successMsg)) {
			echo "<h2 class=\"orange\">Success!</h2><div class=\"basicMedium boldText\">$successMsg</div>";
		}
		
		//Loop through each patient under the account
		$num = 0;
		
		//Only list patients under the account if > 1
		if (isset($patients) && count($patients) > 1) {
			echo "<h2 class=\"orange\">Choose Patient</h2>\n<div class=\"spaced\">";
			foreach ($patients as $patient) {
				echo "<div class=\"basicMedium boldText\">";
				if ($num == $patientNum) {
					//This is the chosen patient; just show their name
					echo "-&gt;" . $fnf->format($patient);
				} else {
					//This is another patient, allow user to choose them
					echo "<a class=\"basicLink boldLink\" href=\"" . $_SERVER['PHP_SELF'] . "?patientNum=$num\">" . $fnf->format($patient) . "</a>";
				}
				echo "</div>\n";
				$num++;
			}
			echo "</div><br />\n";
		}
		
		if (isset($chosenPatient)) {
			?>
			<div id="examsDiv">
				<h2 class="orange">Choose an Exam</h2>
                
                
				<?
				$msg = new GetExamListMessage('cec_portal');
				$exams = $msg->getExamList($chosenPatient->getFieldValue('uid'), $mainPatient->getFieldValue('uid'));
				$baseExams = array();
				
				function isTrue($fieldVal) {
					return ($fieldVal === 'TRUE' || $fieldVal === 'true' || $fieldVal === true);
				}
				//Find the exams that are actually office visits
				if ($exams != null) {
					foreach ($exams as $exam) {
						$cpt = $exam->getFieldValue('cpt');
						$baseExam = $exam->getFieldValue('base_exam');
						$deleted = $exam->getFieldValue('deleted');
						if (isTrue($baseExam) && !isTrue($deleted) && $cpt != null && strtolower($cpt[0]) != 'z' && strlen($cpt) >= 3 && strtolower(substr($cpt, 0, 3)) != '000') {
							//It's an actual office visit
							$baseExams[] = $exam;
						}
					}
				}
				//Show each base exam as a radio button.
				if (count($baseExams) > 0) {
					//Sort by ascending date
					$comparator = new DataHolderFieldComparator(array('date'));
					usort($baseExams, $comparator->getCallback());
					foreach ($baseExams as $exam) {
						$provider = getProviderByUID($exam->getFieldValue('provider'));
						$location = getLocationByUID($exam->getFieldValue('location'));
						$date = date('m/d/Y', strtotime($exam->getFieldValue('date')));
						$uid = $exam->getFieldValue('uid');
						
						echo "<div><input type=\"radio\" name=\"examUID\" class=\"examOption\" value=\"$uid\" id=\"exam_$uid\" date=\"$date\" />";
						echo "<label for=\"exam_$uid\">Exam on $date with Dr. " . $fnf->format($provider) . " at " . $location->getFieldValue('name') . "</label></div>\n";
					}
				} else {
					echo "<div class=\"basicInfo\">You do not have any exams recorded with us.</div>";
				}
				?>
			</div>
              <div id="summary-wrapper">  
            
            
			<div id="clinicalSummaryContent"></div>
			<script type="text/javascript">
				var portalPatient = '<? echo $mainPatient->getFieldValue('uid'); ?>';
				var patName = '<? echo $fnf->format($chosenPatient) ?>';
				var patNum = <? echo $patientNum; ?>;
				
				$(".examOption").click(function() {
					chooseExam();
				});
				
				function chooseExam() {
					var examUID = $(".examOption:checked").val();
					var examDate = $(".examOption:checked").attr('date');
					if (examUID != null) {
						//An exam is chosen.  Load the clinical summary.
						var args = { examUID: examUID, date: examDate, portalPatient: portalPatient, patientName: patName, patientNum: patNum };
						$("#clinicalSummaryContent").html('<h4>Loading episode.  Please wait...</h4>');
						$.post("get_clinical_summary.php", args, function(data) {
							$("#clinicalSummaryContent").html(data);
						});
					}
				}
			</script>
			</div><?
		} else {
			echo "<div class=\"bigorange spaced\">Clinical summary not found</div>\n";
			echo "<div class=\"smallItem\">Your clinical summary could not be found at this time. Please try again later.</div>";
		}
		
		include "foot.php";
	} //end if logged in
} //End if load page
?>