<?
session_start();
require_once('includes/portalutils.php');
require_once('includes/welcome_sheet_utils.php');
require_once(GLOBAL_INCLUDES_DIR . 'xml_to_html.php');
require_once(GLOBAL_INCLUDES_DIR . 'xml_utils.php');
require_once(GLOBAL_INCLUDES_DIR . '/springfield/backend/axis/getwelcomesheetmessage.php');
require_once(GLOBAL_INCLUDES_DIR . '/springfield/backend/axis/updatewelcomesheetmessage.php');

//$patUID = $_SESSION['WelcomeSheetPatient'];
//Don't allow the user to submit the same welcome sheet multiple times.
$map = $_POST;
if ($_GET['useSession'] == 1 && $_SESSION['welcomeSheetPost'] != null) {
	$map = $_SESSION['welcomeSheetPost'];
}
$shouldSubmit = false;
if ($_SESSION['WelcomeSheetSubmit'] == 1) {
	$shouldSubmit = true;
}
$patUID = $map['patient'];
if (isset($patUID) && $patUID != '') {
	$patient = getPatientFromDB(DEFAULT_SERVER, $patUID, false, null);
	if ($patient->getFieldValue("firstname") == null) {
		//No patient was actually loaded - set the patient to null
		$patient = null;
	}
}

//Get the XML file from the post values
$xmlFile = $map['xml_file'];

//Include the header info
$title='EyeCare Associates - Completed Welcome Sheet';
include("template.php");

$drawArrow = false;
$arrowPos = 0;
$subMenu = "portal_menu.jpg";
$rightPadding = 300;
include("head.php");

//Build the enumeration choices
$enumChoices = buildWelcomeSheetEnumerations();

//Get the XML document with field values substituted as named text fields' text
$welcomeSheetStr = null;
$error = null;
$messageTime = null;
if ($xmlFile != null) {
	$doc = getDOMFromFile($xmlFile);
	$textMap = $map;
	//Get the defaults, if any
	$defaults = array();
	foreach ($map as $field => $value) {
		if (strlen($field) > 13 && substr($field, -13) == "_defaultValue") {
			//It's a default value - get the default into the defaults map
			$defaults[substr($field, 0, -13)] = $value;
		}
	}
	$leftUnchanged = array();
	if (count($defaults) > 0) {
		foreach ($defaults as $field => $value) {
			if ($field == 'patient__preferred_name' || $field == 'patient__ssn' || $field == 'patient__employer' || $field == 'patient__cell' ||
					$field == 'patient__workphone' || $field == 'patient__occupation' || $field == 'patient__preferred_language') {
				//These fields can be matched directly
				if (array_key_exists($field, $map) && ($map[$field] == $value || ($map[$field] == '' && $value == null) || ($map[$field] == null && $value == ''))) {
					$leftUnchanged[] = $field;
				}
			} else if ('patient__smoking_status' == $field) {
				if (!hasAnyXs($map, 'patient__SmokingStatus')) {
					$leftUnchanged[] = $field;
				}
			} else if ('patient__married' == $field || 'patient__ethnicity' == $field || 'patient__race' == $field) {
				if (!hasAnyXs($map, $field)) {
					$leftUnchanged[] = $field;
				}
			}
		}
	}
	
	//Fill each unchanged field from the loaded welcome sheet so the patient's data in those fields isn't altered.
	if (count($leftUnchanged) > 0) {
		$msg = new GetWelcomeSheetMessage('cec_welcomesheet');
		try {
			$welcomeSheet = $msg->getWelcomeSheet($patUID);
			foreach ($leftUnchanged as $fieldName) {
				$welcomeSheetName = $fieldName;
				if (strpos($welcomeSheetName, "patient__") == 0) {
					$welcomeSheetName = substr($welcomeSheetName, 9);
				}
				$textMap[$fieldName] = formatPatientField($welcomeSheet, $welcomeSheetName);
				//Special cases must be set as "X"s in the appropriate fields.
				switch ($fieldName) {
					case "patient__smoking_status":
						$textMap = setWelcomeSheetSmokingStatus($textMap, $welcomeSheet->getFieldValue('smoking_status'));
						break;
					case "patient__race":
						$textMap = setWelcomeSheetRace($textMap, $welcomeSheet->getFieldValue('race'));
						break;
					case "patient__ethnicity":
						$textMap = setWelcomeSheetEthnicity($textMap, $welcomeSheet->getFieldValue('ethnicity'));
						break;
					case "patient__married":
						$textMap = setWelcomeSheetMarried($textMap, $welcomeSheet->getFieldValue('married'));
						break;
				}
			}
		} catch (Exception $e) {
			$error = "An unknown error occurred while loading your welcome sheet.  Please try again.";
		}
	}
	
	if ($error == null) {
		$keyMap = convertToISODates($textMap, $doc);
		$keyMap = array_merge($keyMap, getEnumerationKeys($textMap, $doc));
		$textMap = replaceEnumerationText($textMap, $doc, $enumChoices);
		//TODO - REMOVE THIS temporary line (following comment and the line after it)!
		//Temporarily replace all font sizes with one 80% of the size
		$doc = shrinkFonts($doc, 0.85);
		$doc = mapToXML($keyMap, $textMap, $doc);
		
		$welcomeSheetStr = $doc->saveXML();
		//klog('Welcome sheet dump: ' . $welcomeSheetStr, KLogger::ERROR);
	}
}

//TODO - Remove the "&& false" in the statement below.  Included because saving welcome sheets is currently down.
if ($error == null && $shouldSubmit) {
	//Send the resulting XML to the backend for processing
	$msg = new UpdateWelcomeSheetMessage('cec_welcomesheet');
	$lastSlash = strrpos($xmlFile, "/");
	$fileName = ($lastSlash === false ? $xmlFile : substr($xmlFile, $lastSlash + 1));
	if ($patUID != null && $welcomeSheetStr != null) {
		try {
			//TODO - REMOVE THIS temporary line (following comment and the line after it)!
			//Temporarily replace all "Times New Roman" with "Serif" before sending to the backend
			$welcomeSheetStr = str_replace("Times New Roman", "Lucida Sans Regular", $welcomeSheetStr);
			$messageTime = time();
			//klog("Saving welcome sheet:\n" . $welcomeSheetStr, KLogger::ERROR);
			$updateResult = $msg->updateWelcomeSheet($patUID, $fileName, $welcomeSheetStr);
		} catch (Exception $e) {
			//The result being null is sufficient.  It will trigger the error page.
		}
	} else {
		klog('No patient selected when trying to submit welcome sheet', KLogger::ERROR);
	}
}

$errorEmail = TEST_MAIL_TO;
if (isset($updateResult) && $updateResult) {
	//Don't let the user submit again.
	unset($_SESSION['WelcomeSheetSubmit']);
	//If the result is true, the welcome sheet was successfully updated.
	echo "<h3 class=\"orange\">Success!</h3>";
	echo "<p class=\"menuItem\">Your welcome sheet was successfully submitted!  Shown below is a copy of what it looks like in your record.</p>";
	echo xmlFileToHTML($xmlFile, false, '../Imagess/', $map, $enumChoices);
	//If set to do so, email the success to the error mail receiver.
	if ($_GET['emailResult'] == 1) {
		//Calculate elapsed time.
		$elapsedStr = "";
		if ($_SESSION['startTime'] != null && $messageTime != null) {
			$elapsed1 = $messageTime - $_SESSION['startTime'];
			$nowTime = time();
			$elapsed2 = $nowTime - $messageTime;
			$elapsedStr = "\n\n              Time elapsed from start to message send: $elapsed1 seconds\n" .
					"   Time elapsed from message send to receive response: $elapsed2 seconds\n" .
					"Total time from pressing 'Submit' to message response: " . ($elapsed1 + $elapsed2) . " seconds";
		}
		$headers = unserialize(TEST_MAIL_HEADER_ARRAY);
		$headers['Subject'] = 'Test welcome sheet sent successfully';
		if (!pearMail(SMTP_HOST, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD, $errorEmail, 'Test welcome sheet submitted successfully' . $elapsedStr, $headers)) {
			//Log the mail failure
			klog("Error sending 'update welcome sheet test' success email to $errorEmail", KLogger::ERROR);
		}
	}
} else if ($error == null && !$shouldSubmit) {
	echo "<h3 class=\"orange\">Already Sent</h3>";
	echo "<p class=\"menuItem\">Your welcome sheet was already sent successfully.  Please do not refresh this page.  Shown below is a copy of what it looks like in your record.</p>";
	echo xmlFileToHTML($xmlFile, false, '../Images/', $map, $enumChoices);
} else {
	?>

	<script type="text/javascript">
		function printableVersion() {
			$("#printableWelcomeSheet").submit();
		}
	</script>
	
	<?
	//Add the printable welcome sheet form
	echo '<form id="printableWelcomeSheet" target="_blank" method="post" action="printable_welcome_sheet.php">';
	foreach ($map as $name=>$value) {
		echo '<input type="hidden" name="' . $name . '" value="' . $value . '" />';
	}
	echo '</form>';
	
	//Inform the user of the error
	echo "<h3 class=\"orange\">Error</h3>";
	//TODO - Switch back to the other text when the welcome sheet service is working again.
	/* echo "<div class=\"error\">Sorry, but your welcome sheet could not be successfully submitted.  Please try again.  " .
			"If it still doesn't work, please <a class=\"basicLink\" href=\"\" onclick=\"printableVersion(); return false;\">click here</a> " .
			"to view a printable version of the welcome sheet and bring it in to your visit.</div>"; */
	echo "<div class=\"error\">Sorry, but the welcome sheet service is temporarily unavailable.  Please " .
			"<a class=\"basicLink\" href=\"\" onclick=\"printableVersion(); return false;\">click here</a> " .
			"to view a printable version of the welcome sheet and bring it in to your visit.</div>";
	//If set to do so, email the failure to the error mail receiver.
	if ($_GET['emailResult'] == 1) {
		$headers = unserialize(TEST_MAIL_HEADER_ARRAY);
		$headers['Subject'] = "Test welcome sheet could not be saved";
		if (!pearMail(SMTP_HOST, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD, $errorEmail, "Test welcome sheet could not be saved\r\n$error", headers)) {
			//Log the mail failure
			klog("Error sending 'update welcome sheet test' failure email to $errorEmail\r\n$error", KLogger::ERROR);
		}
	}
}

include("foot.php");
?>